@@ -31,7 +31,7 @@
 
 #include "libbalsa.h"
 #include "misc.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 
 LibBalsaMessageBody *
 libbalsa_message_body_new(LibBalsaMessage * message)
@@ -109,7 +109,11 @@ libbalsa_message_body_extract_embedded_headers(GMimeMessage* msg)
     subj = g_mime_message_get_subject(msg);
     if (subj) {
 	ehdr->subject =
+#if HAVE_GMIME_2_2_5
+	    g_mime_utils_header_decode_text(subj);
+#else
 	    g_mime_utils_header_decode_text((const unsigned char *) subj);
+#endif
 	libbalsa_utf8_sanitize(&ehdr->subject, TRUE, NULL);
     } else 
 	ehdr->subject = g_strdup(_("(No subject)"));
@@ -249,7 +253,7 @@ gchar *
 libbalsa_message_body_get_parameter(LibBalsaMessageBody * body,
 				    const gchar * param)
 {
-    gchar *res;
+    gchar *res = NULL;
 
     g_return_val_if_fail(body != NULL, NULL);
 
@@ -257,7 +261,7 @@ libbalsa_message_body_get_parameter(LibBalsaMessageBody * body,
 	const GMimeContentType *type =
 	    g_mime_object_get_content_type(body->mime_part);
 	res = g_strdup(g_mime_content_type_get_parameter(type, param));
-    } else {
+    } else if (body->content_type) {
 	GMimeContentType *type =
 	    g_mime_content_type_new_from_string(body->content_type);
 	res = g_strdup(g_mime_content_type_get_parameter(type, param));
@@ -274,6 +278,12 @@ libbalsa_message_body_get_parameter(LibBalsaMessageBody * body,
 gboolean
 libbalsa_message_body_save_temporary(LibBalsaMessageBody * body, GError **err)
 {
+    if (!body) {
+        g_set_error(err, LIBBALSA_MAILBOX_ERROR, LIBBALSA_MAILBOX_ACCESS_ERROR,
+                    "NULL message body");
+        return FALSE;
+    }
+
     if (body->temp_filename == NULL) {
 	gint count = 100; /* Magic number, same as in g_mkstemp. */
 
