@@ -45,7 +45,7 @@
 #include "filter-funcs.h"
 #include "filter-private.h"
 #include "misc.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 
 /* from libmutt */
 #define REGCOMP(X,Y,Z) regcomp(X, Y, REG_WORDS|REG_EXTENDED|(Z))
@@ -183,7 +183,8 @@ libbalsa_condition_matches(LibBalsaCondition* cond,
 	    if (!message->mailbox)
 		return FALSE; /* We don't want to match if an error occurred */
             body =
-                content2reply(message, NULL, 0, FALSE, FALSE, NULL, NULL);
+                content2reply(message->body_list, NULL, 0,
+                              FALSE, FALSE, NULL, NULL);
 	    if (body) {
 		if (body->str)
                     match = libbalsa_utf8_strstr(body->str,
@@ -262,6 +263,7 @@ libbalsa_filter_mailbox_messages(LibBalsaFilter * filt,
 #endif /* HAVE_GNOME */
     if (filt->popup_text)
 	libbalsa_information(LIBBALSA_INFORMATION_MESSAGE,
+			     "%s",
 			     filt->popup_text);
 
     libbalsa_lock_mailbox(mailbox);
