@@ -31,7 +31,7 @@
 #include "identity.h"
 #include "information.h"
 #include "libbalsa-conf.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 #include "misc.h"
 
 #if ENABLE_ESMTP
@@ -114,6 +114,7 @@ libbalsa_identity_init(LibBalsaIdentity* ident)
     ident->always_trust = FALSE;
     ident->crypt_protocol = LIBBALSA_PROTECT_OPENPGP;
 #endif
+    ident->request_mdn = FALSE;
     /*
     ident->face = NULL;
     ident->x_face = NULL;
@@ -305,6 +306,72 @@ libbalsa_identity_set_sig_prepend(LibBalsaIdentity* ident, gboolean prepend)
     ident->sig_prepend = prepend;
 }
 
+/** Returns a signature for given identity, adding a signature prefix
+    if needed. parent can be NULL. */
+gchar*
+libbalsa_identity_get_signature(LibBalsaIdentity* identity, GtkWindow *parent)
+{
+    FILE *fp = NULL;
+    size_t len = 0;
+    gchar *ret = NULL, *path;
+
+    if (identity->signature_path == NULL ||
+        *identity->signature_path == '\0')
+	return NULL;
+
+    path = libbalsa_expand_path(identity->signature_path);
+    if(identity->sig_executable){
+        /* signature is executable */
+	fp = popen(path,"r");
+        if (fp) {
+            len = libbalsa_readfile_nostat(fp, &ret);
+            pclose(fp);
+        } else
+            libbalsa_information_parented
+                (parent, LIBBALSA_INFORMATION_ERROR,
+                 _("Error executing signature generator %s"),
+                 identity->signature_path);
+    } else {
+        /* sign is normal file */
+        fp = fopen(path, "r");
+        if (fp) {
+            len = libbalsa_readfile_nostat(fp, &ret);
+            fclose(fp);
+        } else
+            libbalsa_information_parented(parent, LIBBALSA_INFORMATION_ERROR,
+                                          _("Cannot open signature file '%s' "
+                                            "for reading"),
+                                          identity->signature_path);
+    }
+    if(!ret)
+        libbalsa_information_parented(parent, LIBBALSA_INFORMATION_ERROR,
+                                   _("Error reading signature from %s"), path);
+    else {
+        if(!libbalsa_utf8_sanitize(&ret, FALSE, NULL))
+            libbalsa_information_parented
+                (parent, LIBBALSA_INFORMATION_ERROR,
+                 _("Signature in %s is not a UTF-8 text."),
+                 identity->signature_path);
+    }
+    g_free(path);
+
+    if(ret == NULL) return NULL;
+
+    /* Prepend the separator if needed... */
+
+    if (identity->sig_separator
+        && strncmp(ret, "--\n", 3)
+        && strncmp(ret, "-- \n", 4)) {
+        gchar *sig_tmp = g_strconcat("\n-- \n", ret, NULL);
+        g_free(ret);
+        ret = sig_tmp;
+    } else {
+        gchar *sig_tmp = g_strconcat("\n", ret, NULL);
+        g_free(ret);
+        ret = sig_tmp;
+    }
+    return ret;
+}
 
 #if ENABLE_ESMTP
 void
@@ -556,11 +623,11 @@ static void md_response_cb(GtkWidget * dialog, gint response,
                            GtkTreeView * tree);
 static void md_name_changed(GtkEntry * name, GtkTreeView * tree);
 
-#ifdef HAVE_GPGME
 static void ident_dialog_add_gpg_menu(GtkWidget * table, gint row,
                                       GtkDialog * dialog,
                                       const gchar * label_name,
                                       const gchar * menu_key);
+#ifdef HAVE_GPGME
 static void display_frame_set_gpg_mode(GObject * dialog,
                                        const gchar * key, gint * value);
 #endif /* HAVE_GPGME */
@@ -773,10 +840,10 @@ get_selected_identity(GtkTreeView * tree)
 }
 
 enum {
+    IDENTITY_RESPONSE_HELP = GTK_RESPONSE_HELP,
     IDENTITY_RESPONSE_CLOSE,
     IDENTITY_RESPONSE_NEW,
-    IDENTITY_RESPONSE_REMOVE,
-    IDENTITY_RESPONSE_HELP
+    IDENTITY_RESPONSE_REMOVE
 };
     
 /* callback for the "changed" signal */
@@ -834,7 +901,8 @@ new_ident_cb(GtkTreeView * tree, GObject * dialog)
  */
 static GtkWidget*
 append_ident_notebook_page(GtkNotebook *notebook, guint rows,
-			   const gchar * tab_label)
+			   const gchar * tab_label,
+                           const gchar * footnote)
 {
     GtkWidget *vbox;
     GtkWidget *table;
@@ -843,6 +911,9 @@ append_ident_notebook_page(GtkNotebook *notebook, guint rows,
     table = libbalsa_create_table(rows, 2);
     gtk_container_set_border_width(GTK_CONTAINER(table), padding);
     gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
+    if (footnote)
+        gtk_box_pack_start(GTK_BOX(vbox), gtk_label_new(footnote),
+                           FALSE, FALSE, 0);
     gtk_notebook_append_page(notebook, vbox, gtk_label_new(tab_label));
     
     return table;
@@ -891,9 +962,10 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
     gint row;
     GObject *name;
     gpointer path;
+    gchar *footnote;
 
     /* create the "General" tab */
-    table = append_ident_notebook_page(notebook, 8, _("General"));
+    table = append_ident_notebook_page(notebook, 5, _("General"), NULL);
     row = 0;
     ident_dialog_add_entry(table, row++, dialog, _("_Identity Name:"), 
 		           "identity-name");
@@ -905,19 +977,33 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
                            "identity-replyto");
     ident_dialog_add_entry(table, row++, dialog, _("_Domain:"), 
                            "identity-domain");
+
+    /* create the "Messages" tab */
+    table = append_ident_notebook_page(notebook, 8, _("Messages"), NULL);
+    row = 0;
     ident_dialog_add_entry(table, row++, dialog, _("_Bcc:"), 
                            "identity-bcc");
     ident_dialog_add_entry(table, row++, dialog, _("Reply _String:"), 
                            "identity-replystring");
     ident_dialog_add_entry(table, row++, dialog, _("F_orward String:"), 
                            "identity-forwardstring");
+    ident_dialog_add_checkbutton(table, row++, dialog, 
+                                 _("request _Message Disposition Notification by default"),
+                                 "identity-requestmdn", TRUE);
+    ident_dialog_add_file_chooser_button(table, row++, dialog,
+                                         LBI_PATH_TYPE_FACE);
+    ident_dialog_add_file_chooser_button(table, row++, dialog,
+                                         LBI_PATH_TYPE_XFACE);
+    ident_dialog_add_boxes(table, row++, dialog,
+                           path_info[LBI_PATH_TYPE_FACE].box_key,
+                           path_info[LBI_PATH_TYPE_XFACE].box_key);
 #if ENABLE_ESMTP
     ident_dialog_add_smtp_menu(table, row++, dialog, _("SMT_P Server:"),
                                "identity-smtp-server", smtp_servers);
 #endif /* ENABLE_ESMTP */
 
     /* create the "Signature" tab */
-    table = append_ident_notebook_page(notebook, 7, _("Signature"));
+    table = append_ident_notebook_page(notebook, 7, _("Signature"), NULL);
     row = 0;
     ident_dialog_add_check_and_entry(table, row++, dialog,
                                      _("Signature _Path"),
@@ -940,17 +1026,16 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
     ident_dialog_add_checkbutton(table, row++, dialog,
                                  _("Prepend Si_gnature"),
                                  "identity-sigprepend", FALSE);
-    ident_dialog_add_file_chooser_button(table, row++, dialog,
-                                         LBI_PATH_TYPE_FACE);
-    ident_dialog_add_file_chooser_button(table, row++, dialog,
-                                         LBI_PATH_TYPE_XFACE);
-    ident_dialog_add_boxes(table, row++, dialog,
-                           path_info[LBI_PATH_TYPE_FACE].box_key,
-                           path_info[LBI_PATH_TYPE_XFACE].box_key);
 
 #ifdef HAVE_GPGME
+    footnote = NULL;
+#else
+    footnote = _("Signing and encrypting messages are possible\n"
+                 "only if Balsa is built with cryptographic support.");
+#endif
     /* create the "Security" tab */
-    table = append_ident_notebook_page(notebook, 4, _("Security"));
+    table =
+        append_ident_notebook_page(notebook, 4, _("Security"), footnote);
     row = 0;
     ident_dialog_add_checkbutton(table, row++, dialog, 
                                  _("sign messages by default"),
@@ -964,6 +1049,8 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
     ident_dialog_add_checkbutton(table, row++, dialog,
                                  _("always trust GnuPG keys when encrypting"),
                                  "identity-trust-always", TRUE);
+#ifndef HAVE_GPGME
+    gtk_widget_set_sensitive(table, FALSE);
 #endif
 
     name = g_object_get_data(G_OBJECT(dialog), "identity-name");
@@ -1111,7 +1198,7 @@ md_face_path_changed(const gchar * filename, gboolean active,
         libbalsa_information(LIBBALSA_INFORMATION_WARNING,
                 /* Translators: please do not translate Face. */
                              _("Face header file %s is too long "
-                               "(%d bytes)."), filename, size);
+                               "(%lu bytes)."), filename, (unsigned long)size);
         g_free(content);
         gtk_widget_hide(face_box);
         return;
@@ -1377,7 +1464,7 @@ ident_dialog_update(GObject * dlg)
     g_free(id->forward_string);
     id->forward_string  = ident_dialog_get_text(dlg, "identity-forwardstring");
 #if ENABLE_ESMTP
-    g_object_unref(id->smtp_server);
+    if(id->smtp_server) g_object_unref(id->smtp_server);
     id->smtp_server = ident_dialog_get_value(dlg, "identity-smtp-server");
     g_object_ref(id->smtp_server);
 #endif /* ENABLE_ESMTP */
@@ -1396,7 +1483,8 @@ ident_dialog_update(GObject * dlg)
     id->face            = ident_dialog_get_path(dlg, "identity-facepath");
     g_free(id->x_face);
     id->x_face          = ident_dialog_get_path(dlg, "identity-xfacepath");
-    
+    id->request_mdn     = ident_dialog_get_bool(dlg, "identity-requestmdn");
+
 #ifdef HAVE_GPGME
     id->gpg_sign        = ident_dialog_get_bool(dlg, "identity-gpgsign");
     id->gpg_encrypt     = ident_dialog_get_bool(dlg, "identity-gpgencrypt");
@@ -1771,6 +1859,8 @@ display_frame_update(GObject * dialog, LibBalsaIdentity* ident)
     gtk_widget_hide(face_box);
     display_frame_set_path(dialog, path_info[LBI_PATH_TYPE_XFACE].path_key,
                            ident->x_face, TRUE);
+    display_frame_set_boolean(dialog, "identity-requestmdn", 
+                              ident->request_mdn);    
 
 #ifdef HAVE_GPGME
     display_frame_set_boolean(dialog, "identity-gpgsign", 
@@ -1873,6 +1963,7 @@ libbalsa_identity_new_config(const gchar* name)
     ident->sig_prepend = libbalsa_conf_get_bool("SigPrepend");
     ident->face = libbalsa_conf_get_string("FacePath");
     ident->x_face = libbalsa_conf_get_string("XFacePath");
+    ident->request_mdn = libbalsa_conf_get_bool("RequestMDN");
 
 #ifdef HAVE_GPGME
     ident->gpg_sign = libbalsa_conf_get_bool("GpgSign");
@@ -1917,6 +2008,7 @@ libbalsa_identity_save(LibBalsaIdentity* ident, const gchar* group)
         libbalsa_conf_set_string("FacePath", ident->face);
     if (ident->x_face)
         libbalsa_conf_set_string("XFacePath", ident->x_face);
+    libbalsa_conf_set_bool("RequestMDN", ident->request_mdn);
 
 #ifdef HAVE_GPGME
     libbalsa_conf_set_bool("GpgSign", ident->gpg_sign);
@@ -1956,12 +2048,41 @@ libbalsa_identity_set_crypt_protocol(LibBalsaIdentity* ident, gint protocol)
 }
 
 
+
+static void
+display_frame_set_gpg_mode(GObject * dialog, const gchar* key, gint * value)
+{
+    GtkComboBox *opt_menu = g_object_get_data(G_OBJECT(dialog), key);
+ 
+    switch (*value)
+        {
+        case LIBBALSA_PROTECT_OPENPGP:
+	    gtk_combo_box_set_active(opt_menu, 1);
+            break;
+#ifdef HAVE_SMIME
+        case LIBBALSA_PROTECT_SMIMEV3:
+	    gtk_combo_box_set_active(opt_menu, 2);
+            break;
+#endif
+        case LIBBALSA_PROTECT_RFC3156:
+        default:
+	    gtk_combo_box_set_active(opt_menu, 0);
+            *value = LIBBALSA_PROTECT_RFC3156;
+        }
+}
+#endif  /* HAVE_GPGME */
+
 /*
  * Add an option menu to the given dialog with a label next to it
  * explaining the contents.  A reference to the entry is stored as
  * object data attached to the dialog with the given key.
  */
 
+#ifndef HAVE_GPGME
+/* So we can build a dummy Security page: */
+#define LIBBALSA_PROTECT_RFC3156 0
+#define LIBBALSA_PROTECT_OPENPGP 0
+#endif
 static void
 ident_dialog_add_gpg_menu(GtkWidget * table, gint row, GtkDialog * dialog,
                           const gchar * label_name, const gchar * menu_key)
@@ -1991,30 +2112,6 @@ ident_dialog_add_gpg_menu(GtkWidget * table, gint row, GtkDialog * dialog,
 #endif
 }
 
-
-static void
-display_frame_set_gpg_mode(GObject * dialog, const gchar* key, gint * value)
-{
-    GtkComboBox *opt_menu = g_object_get_data(G_OBJECT(dialog), key);
- 
-    switch (*value)
-        {
-        case LIBBALSA_PROTECT_OPENPGP:
-	    gtk_combo_box_set_active(opt_menu, 1);
-            break;
-#ifdef HAVE_SMIME
-        case LIBBALSA_PROTECT_SMIMEV3:
-	    gtk_combo_box_set_active(opt_menu, 2);
-            break;
-#endif
-        case LIBBALSA_PROTECT_RFC3156:
-        default:
-	    gtk_combo_box_set_active(opt_menu, 0);
-            *value = LIBBALSA_PROTECT_RFC3156;
-        }
-}
-#endif  /* HAVE_GPGME */
-
 #if ENABLE_ESMTP
 static void
 ident_dialog_add_smtp_menu(GtkWidget * table, gint row, GtkDialog * dialog,
