@@ -920,22 +920,30 @@ write_nstring(unsigned seqno, const char *str, int len, void *fl)
 }
 
 ImapResponse
-imap_mbox_handle_fetch_rfc822(ImapMboxHandle* handle, unsigned seqno, 
+imap_mbox_handle_fetch_rfc822(ImapMboxHandle* handle,
+			      unsigned cnt, unsigned *set,
                               FILE *fl)
 {
-  char cmd[40];
-  ImapFetchBodyCb cb = handle->body_cb;
-  void          *arg = handle->body_arg;
-  ImapResponse rc;
+  ImapResponse rc = IMR_OK;
+  gchar *seq;
 
   IMAP_REQUIRED_STATE1(handle, IMHS_SELECTED, IMR_BAD);
   HANDLE_LOCK(handle);
-  handle->body_cb  = write_nstring;
-  handle->body_arg = fl;
-  sprintf(cmd, "FETCH %u RFC822", seqno);
-  rc = imap_cmd_exec(handle, cmd);
-  handle->body_cb  = cb;
-  handle->body_arg = arg;
+
+  seq = coalesce_seq_range(0, cnt-1, (CoalesceFunc)simple_coealesce_func, set);
+
+  if(seq) {
+    ImapFetchBodyCb cb = handle->body_cb;
+    void          *arg = handle->body_arg;
+    gchar *cmd = g_strdup_printf("FETCH %s RFC822", seq);
+    handle->body_cb  = write_nstring;
+    handle->body_arg = fl;
+    rc = imap_cmd_exec(handle, cmd);
+    handle->body_cb  = cb;
+    handle->body_arg = arg;
+    g_free(cmd);
+    g_free(seq);
+  }
   HANDLE_UNLOCK(handle);
 
   return rc;
@@ -1012,6 +1020,7 @@ imap_mbox_handle_fetch_body(ImapMboxHandle* handle,
     ibd.body = imap_message_get_body_from_section(imsg, section);
     ibd.body_cb = body_cb;
     ibd.body_arg = arg;
+    ibd.first_run = TRUE;
     handle->body_cb = imap_binary_handler;
     handle->body_arg = &ibd;
     snprintf(cmd, sizeof(cmd), "FETCH %u BINARY[%s]",
