@@ -104,12 +104,15 @@ struct _ImapMboxHandle {
                            inactivity */
   guint async_watch_id;  /* callback to process incoming data */
   ImapTlsMode tls_mode; /* disabled, enabled, required */
+  unsigned op_cancelled:1; /* last op timed out and was cancelled by user */
   unsigned idle_issued:1; /*  idle has been issued */
   unsigned readonly_mbox:1;
   unsigned can_fetch_body:1; /* set for servers that always respond
                               * correctly to FETCH x BODY[y]
                               * requests. */
   unsigned enable_anonymous:1; /* try anonymous if possible */
+  unsigned enable_client_sort:1; /**< client side sorting allowed */
+  unsigned enable_idle:1;    /**< use IDLE - no problem with firewalls */
   unsigned enable_binary:1; /* enable binary extension */
 
   /* conditional stuff at the end for the safety. */
