@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2000 Stuart Parmenter and others,
+ * Copyright (C) 1997-2007 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,12 +23,18 @@
 #include "config.h"
 
 #include "information.h"
+#ifdef HAVE_NOTIFY
+#include <libnotify/notify.h>
+#include <gtk/gtkstock.h>
+#endif
+#include <string.h>
 
 struct information_data {
     GtkWindow *parent;
     LibBalsaInformationType message_type;
     gchar *msg;
 };
+
 static gboolean libbalsa_information_idle_handler(struct information_data*);
 
 LibBalsaInformationFunc libbalsa_real_information_func;
@@ -47,20 +53,72 @@ void
 libbalsa_information_varg(GtkWindow *parent, LibBalsaInformationType type,
                           const char *fmt, va_list ap)
 {
-    struct information_data *data = g_new(struct information_data, 1);
+    struct information_data *data;
 
     g_return_if_fail(fmt != NULL);
     g_assert(libbalsa_real_information_func != NULL);
 
+#ifdef HAVE_NOTIFY
+    if (notify_is_initted()) {
+        gchar *msg, *p, *q;
+        GString *escaped;
+        NotifyNotification *note;
+        char *icon_str;
+
+        switch (type) {
+        case LIBBALSA_INFORMATION_MESSAGE:
+            icon_str = GTK_STOCK_DIALOG_INFO;
+            break;
+        case LIBBALSA_INFORMATION_WARNING:
+            icon_str = GTK_STOCK_DIALOG_WARNING;
+            break;
+        case LIBBALSA_INFORMATION_ERROR:
+            icon_str = GTK_STOCK_DIALOG_ERROR;
+            break;
+        default:
+            icon_str = NULL;
+            break;
+        }
+        msg = g_strdup_vprintf(fmt, ap);
+        /* libnotify/DBUS uses HTML markup, so we must replace '<' and
+         * '&' with the corresponding entity in the message string. */
+        escaped = g_string_new(NULL);
+        for (p = msg; (q = strpbrk(p, "<>&\"")) != NULL; p = ++q) {
+            g_string_append_len(escaped, p, q - p);
+            switch (*q) {
+                case '<': g_string_append(escaped, "&lt;");   break;
+                case '>': g_string_append(escaped, "&gt;");   break;
+                case '&': g_string_append(escaped, "&amp;");  break;
+                case '"': g_string_append(escaped, "&quot;"); break;
+                default: break;
+            }
+        }
+        g_string_append(escaped, p);
+        g_free(msg);
+
+        note =
+            notify_notification_new("Balsa", escaped->str, icon_str, NULL);
+        g_string_free(escaped, TRUE);
+
+        notify_notification_set_timeout(note, 7000);    /* 7 seconds */
+        notify_notification_show(note, NULL);
+        g_object_unref(note);
+        return;
+    }
+    /* Fall through to the ordinary notification scheme */
+#endif
+    data = g_new(struct information_data, 1);
+    data->parent = parent;
+    data->message_type = type;
+
     /* We format the string here. It must be free()d in the idle
      * handler We parse the args here because by the time the idle
      * function runs we will no longer be in this stack frame. 
      */
-    data->parent = parent;
-    data->message_type = type;
     data->msg = g_strdup_vprintf(fmt, ap);
-    if(parent)
-        g_object_add_weak_pointer(G_OBJECT(parent), (gpointer) &data->parent);
+    if (parent)
+        g_object_add_weak_pointer(G_OBJECT(parent),
+                                  (gpointer) & data->parent);
     g_idle_add((GSourceFunc) libbalsa_information_idle_handler, data);
 }
 
@@ -70,6 +128,9 @@ libbalsa_information(LibBalsaInformationType type,
 {
     va_list va_args;
 
+#ifndef DEBUG
+    if (type == LIBBALSA_INFORMATION_DEBUG) return;
+#endif
     va_start(va_args, fmt);
     libbalsa_information_varg(NULL, type, fmt, va_args);
     va_end(va_args);
