@@ -54,7 +54,7 @@
 
 #include "libbalsa.h"
 #include "misc.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 
 #ifdef BALSA_USE_THREADS
 static pthread_t main_thread_id;
@@ -113,6 +113,9 @@ libbalsa_init(LibBalsaInformationFunc information_callback)
 #if HAVE_SQLITE
     LIBBALSA_TYPE_ADDRESS_BOOK_GPE;
 #endif
+#if HAVE_RUBRICA
+    LIBBALSA_TYPE_ADDRESS_BOOK_RUBRICA;
+#endif
 }
 
 
@@ -614,8 +617,9 @@ ask_timeout_real(void *data)
 
 gboolean
 libbalsa_abort_on_timeout(const char *host)
-{
-    return libbalsa_ask(ask_timeout_real, (void*)host) != 0;
+{  /* It appears not to be entirely thread safe... Some locks do not
+      get released as they should be. */
+    return libbalsa_ask(ask_timeout_real, (void*)host) != 0; 
 }
 
 
