@@ -34,7 +34,7 @@
 #include "filter-funcs.h"
 #include "mailbox-filter.h"
 #include "misc.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 
 
 static LibBalsaMailboxClass *parent_class = NULL;
@@ -50,6 +50,9 @@ static void libbalsa_mailbox_local_load_config(LibBalsaMailbox * mailbox,
 
 static void libbalsa_mailbox_local_close_mailbox(LibBalsaMailbox * mailbox,
                                                  gboolean expunge);
+static LibBalsaMessage *libbalsa_mailbox_local_get_message(LibBalsaMailbox
+                                                           * mailbox,
+                                                           guint msgno);
 static gboolean libbalsa_mailbox_local_message_match(LibBalsaMailbox *
 						     mailbox, guint msgno,
 						     LibBalsaMailboxSearchIter
@@ -61,9 +64,9 @@ static void libbalsa_mailbox_local_set_threading(LibBalsaMailbox *mailbox,
 static void lbm_local_update_view_filter(LibBalsaMailbox * mailbox,
                                          LibBalsaCondition *view_filter);
 
-static void libbalsa_mailbox_local_prepare_threading(LibBalsaMailbox *mailbox, 
-                                                     guint * msgnos,
-                                                     guint len);
+static gboolean libbalsa_mailbox_local_prepare_threading(LibBalsaMailbox *
+                                                         mailbox,
+                                                         guint start);
 
 static gboolean libbalsa_mailbox_local_fetch_structure(LibBalsaMailbox *
                                                        mailbox,
@@ -81,6 +84,18 @@ static gboolean libbalsa_mailbox_local_get_msg_part(LibBalsaMessage *msg,
 static void lbm_local_sort(LibBalsaMailbox * mailbox, GArray *sort_array);
 static GArray *libbalsa_mailbox_local_duplicate_msgnos(LibBalsaMailbox *
                                                        mailbox);
+static gboolean
+libbalsa_mailbox_local_messages_change_flags(LibBalsaMailbox * mailbox,
+                                             GArray * msgnos,
+                                             LibBalsaMessageFlag set,
+                                             LibBalsaMessageFlag clear);
+static gboolean libbalsa_mailbox_local_msgno_has_flags(LibBalsaMailbox *
+                                                       mailbox,
+                                                       guint msgno,
+                                                       LibBalsaMessageFlag
+                                                       set,
+                                                       LibBalsaMessageFlag
+                                                       unset);
 
 /* LibBalsaMailboxLocal class method: */
 static void lbm_local_real_remove_files(LibBalsaMailboxLocal * local);
@@ -132,6 +147,8 @@ libbalsa_mailbox_local_class_init(LibBalsaMailboxLocalClass * klass)
 
     libbalsa_mailbox_class->close_mailbox =
 	libbalsa_mailbox_local_close_mailbox;
+    libbalsa_mailbox_class->get_message =
+	libbalsa_mailbox_local_get_message;
     libbalsa_mailbox_class->message_match = 
         libbalsa_mailbox_local_message_match;
     libbalsa_mailbox_class->set_threading =
@@ -147,9 +164,12 @@ libbalsa_mailbox_local_class_init(LibBalsaMailboxLocalClass * klass)
     libbalsa_mailbox_class->get_message_part = 
         libbalsa_mailbox_local_get_msg_part;
     libbalsa_mailbox_class->sort = lbm_local_sort;
+    libbalsa_mailbox_class->messages_change_flags =
+        libbalsa_mailbox_local_messages_change_flags;
+    libbalsa_mailbox_class->msgno_has_flags =
+        libbalsa_mailbox_local_msgno_has_flags;
     libbalsa_mailbox_class->duplicate_msgnos =
         libbalsa_mailbox_local_duplicate_msgnos;
-    klass->load_message = NULL;
     klass->check_files  = NULL;
     klass->set_path     = NULL;
     klass->remove_files = lbm_local_real_remove_files;
@@ -237,36 +257,76 @@ libbalsa_mailbox_local_remove_files(LibBalsaMailboxLocal * local)
 
 /* libbalsa_mailbox_load_message:
    MAKE sure the mailbox is LOCKed before entering this routine.
-*/ 
+*/
+
+static void
+lbml_add_message_to_pool(LibBalsaMailboxLocal * local,
+                         LibBalsaMessage * message)
+{
+    LibBalsaMailboxLocalPool *item, *oldest;
+
+    ++local->pool_seqno;
+
+    for (item = oldest = &local->message_pool[0];
+         item < &local->message_pool[LBML_POOL_SIZE]; item++) {
+        if (item->message == message) {
+            item->pool_seqno = local->pool_seqno;
+            return;
+        }
+        if (item->pool_seqno < oldest->pool_seqno)
+            oldest = item;
+    }
+
+    if (oldest->message)
+        g_object_unref(oldest->message);
+    oldest->message = g_object_ref(message);
+    oldest->pool_seqno = local->pool_seqno;
+}
+
+static void
+lbm_local_get_message_with_msg_info(LibBalsaMailboxLocal * local,
+                                    guint msgno,
+                                    LibBalsaMailboxLocalMessageInfo *
+                                    msg_info)
+{
+    LibBalsaMessage *message;
+
+    msg_info->message = message = libbalsa_message_new();
+    g_object_add_weak_pointer(G_OBJECT(message),
+                              (gpointer) & msg_info->message);
+
+    message->flags = msg_info->flags & LIBBALSA_MESSAGE_FLAGS_REAL;
+    message->mailbox = LIBBALSA_MAILBOX(local);
+    message->msgno = msgno;
+    libbalsa_message_load_envelope(message);
+    lbml_add_message_to_pool(local, message);
+}
+
 static gboolean message_match_real(LibBalsaMailbox * mailbox, guint msgno,
                                    LibBalsaCondition * cond);
-static LibBalsaMessage *
+static void
 libbalsa_mailbox_local_load_message(LibBalsaMailboxLocal * local,
-                                    GNode ** sibling, guint msgno)
+                                    GNode ** sibling, guint msgno,
+                                    LibBalsaMailboxLocalMessageInfo *
+                                    msg_info)
 {
-    LibBalsaMessage *msg = NULL;
-    LibBalsaMessageFlag flags;
     LibBalsaMailbox *mbx = LIBBALSA_MAILBOX(local);
     gboolean match;
 
-    flags =
-        LIBBALSA_MAILBOX_LOCAL_GET_CLASS(local)->load_message(local, msgno,
-                                                              &msg);
-
-    if ((flags & LIBBALSA_MESSAGE_FLAG_NEW)
-        && !(flags & LIBBALSA_MESSAGE_FLAG_DELETED)) {
+    if ((msg_info->flags & LIBBALSA_MESSAGE_FLAG_NEW)
+        && !(msg_info->flags & LIBBALSA_MESSAGE_FLAG_DELETED)) {
         mbx->unread_messages++;
         if (mbx->first_unread == 0)
             mbx->first_unread = msgno;
     }
 
-    if (flags & LIBBALSA_MESSAGE_FLAG_RECENT) {
+    if (msg_info->flags & LIBBALSA_MESSAGE_FLAG_RECENT) {
         gchar *id;
 
-        if (!msg)
-            msg = libbalsa_mailbox_get_message(mbx, msgno);
+        if (!msg_info->message)
+            lbm_local_get_message_with_msg_info(local, msgno, msg_info);
 
-        if (libbalsa_message_is_partial(msg, &id)) {
+        if (libbalsa_message_is_partial(msg_info->message, &id)) {
             libbalsa_mailbox_try_reassemble(mbx, id);
             g_free(id);
         }
@@ -281,8 +341,6 @@ libbalsa_mailbox_local_load_message(LibBalsaMailboxLocal * local,
     if (match)
         libbalsa_mailbox_msgno_inserted(mbx, msgno, mbx->msg_tree,
                                         sibling);
-
-    return msg;
 }
 
 /* Threading info. */
@@ -466,7 +524,7 @@ lbm_local_save_tree(LibBalsaMailboxLocal * local)
 
     filename = lbm_local_get_cache_filename(local);
 
-    if (libbalsa_mailbox_get_total(mailbox) == 0
+    if (!mailbox->msg_tree->children
         || (libbalsa_mailbox_get_threading_type(mailbox) ==
             LB_MAILBOX_THREADING_FLAT
             && libbalsa_mailbox_get_sort_field(mailbox) ==
@@ -699,6 +757,8 @@ libbalsa_mailbox_local_close_mailbox(LibBalsaMailbox * mailbox,
                                      gboolean expunge)
 {
     LibBalsaMailboxLocal *local = LIBBALSA_MAILBOX_LOCAL(mailbox);
+    guint i;
+    LibBalsaMailboxLocalPool *item;
 
     if(local->sync_id) {
         g_source_remove(local->sync_id);
@@ -722,13 +782,12 @@ libbalsa_mailbox_local_close_mailbox(LibBalsaMailbox * mailbox,
         /* Save immediately. */
         g_source_remove(local->save_tree_id);
         local->save_tree_id = 0;
-        lbm_local_save_tree(local);
     }
+    lbm_local_save_tree(local);
 
     if (local->threading_info) {
 	/* Free the memory owned by local->threading_info, but neither
 	 * free nor truncate the array. */
-	guint i;
         for (i = local->threading_info->len; i > 0;) {
             gpointer *entry =
                 &g_ptr_array_index(local->threading_info, --i);
@@ -737,11 +796,38 @@ libbalsa_mailbox_local_close_mailbox(LibBalsaMailbox * mailbox,
         }
     }
 
+    for (item = &local->message_pool[0];
+         item < &local->message_pool[LBML_POOL_SIZE]; item++) {
+        if (item->message) {
+            g_object_unref(item->message);
+            item->message = NULL;
+        }
+        item->pool_seqno = 0;
+    }
+    local->pool_seqno = 0;
+
     if (LIBBALSA_MAILBOX_CLASS(parent_class)->close_mailbox)
         LIBBALSA_MAILBOX_CLASS(parent_class)->close_mailbox(mailbox,
                                                             expunge);
 }
 
+/* LibBalsaMailbox get_message class method */
+
+static LibBalsaMessage *
+libbalsa_mailbox_local_get_message(LibBalsaMailbox * mailbox, guint msgno)
+{
+    LibBalsaMailboxLocal *local = LIBBALSA_MAILBOX_LOCAL(mailbox);
+    LibBalsaMailboxLocalMessageInfo *msg_info =
+        LIBBALSA_MAILBOX_LOCAL_GET_CLASS(local)->get_info(local, msgno);
+
+    if (msg_info->message)
+        return g_object_ref(msg_info->message);
+
+    lbm_local_get_message_with_msg_info(local, msgno, msg_info);
+
+    return msg_info->message;
+}
+
 /* Search iters. We do not use the fallback version because it does
    lot of reparsing. Instead, we use LibBalsaMailboxIndex entry
    whenever possible - it is so quite frequently. This is a big
@@ -769,21 +855,33 @@ message_match_real(LibBalsaMailbox *mailbox, guint msgno,
      * fetch it here, and that will also populate entry and info. */
     if (!entry || !info) {
         message = libbalsa_mailbox_get_message(mailbox, msgno);
+        if (!message)
+            return FALSE;
         libbalsa_mailbox_local_cache_message(local, msgno, message);
         entry = g_ptr_array_index(mailbox->mindex, msgno-1);
         info  = g_ptr_array_index(local->threading_info, msgno - 1);
     }
 
+#if defined(BALSA_USE_THREADS)
+    if (entry->idle_pending)
+        return FALSE;   /* Can't match. */
+#endif                  /* defined(BALSA_USE_THREADS) */
+
     switch (cond->type) {
     case CONDITION_STRING:
         if (CONDITION_CHKMATCH
             (cond, (CONDITION_MATCH_CC | CONDITION_MATCH_BODY))) {
-            message = libbalsa_mailbox_get_message(mailbox, msgno);
+            if (!message)
+                message = libbalsa_mailbox_get_message(mailbox, msgno);
+            if (!message)
+                return FALSE;
             is_refed = libbalsa_message_body_ref(message, FALSE, FALSE);
             if (!is_refed) {
                 libbalsa_information(LIBBALSA_INFORMATION_ERROR,
                                      _("Unable to load message body to "
                                        "match filter"));
+                if (message)
+                    g_object_unref(message);
                 return FALSE;   /* We don't want to match if an error occurred */
             }
         }
@@ -792,6 +890,8 @@ message_match_real(LibBalsaMailbox *mailbox, guint msgno,
 	if (CONDITION_CHKMATCH(cond,CONDITION_MATCH_TO)) {
             if (!message)
                 message = libbalsa_mailbox_get_message(mailbox, msgno);
+            if (!message)
+                return FALSE;
             str = internet_address_list_to_string(message->headers->to_list,
                                                   FALSE);
             match = libbalsa_utf8_strstr(str,cond->match.string.string);
@@ -827,6 +927,8 @@ message_match_real(LibBalsaMailbox *mailbox, guint msgno,
 
                 if (!message)
                     message = libbalsa_mailbox_get_message(mailbox, msgno);
+                if (!message)
+                    return FALSE;
                 header =
                     libbalsa_message_get_user_header(message,
                                                      cond->match.string.
@@ -841,10 +943,14 @@ message_match_real(LibBalsaMailbox *mailbox, guint msgno,
 	if (CONDITION_CHKMATCH(cond,CONDITION_MATCH_BODY)) {
             GString *body;
             g_assert(is_refed);
-	    if (!message->mailbox)
+	    if (!message->mailbox) {
+                /* No need to body-unref */
+                g_object_unref(message);
 		return FALSE; /* We don't want to match if an error occurred */
+            }
             body =
-                content2reply(message, NULL, 0, FALSE, FALSE, NULL, NULL);
+                content2reply(message->body_list, NULL, 
+                              0, FALSE, FALSE, NULL, NULL);
 	    if (body) {
 		if (body->str)
                     match = libbalsa_utf8_strstr(body->str,
@@ -875,6 +981,7 @@ message_match_real(LibBalsaMailbox *mailbox, guint msgno,
             message_match_real(mailbox, msgno, cond->match.andor.left) ||
             message_match_real(mailbox, msgno, cond->match.andor.right);
         break;
+    /* To avoid warnings */
     case CONDITION_NONE:
         break;
     }
@@ -882,7 +989,6 @@ message_match_real(LibBalsaMailbox *mailbox, guint msgno,
         if(is_refed) libbalsa_message_body_unref(message);
         g_object_unref(message);
     }
-    /* To avoid warnings */
     return cond->negate ? !match : match;
 }
 static gboolean
@@ -898,6 +1004,10 @@ libbalsa_mailbox_local_message_match(LibBalsaMailbox * mailbox,
  * PS: called by mail_progress_notify_cb:
  * loads incrementally new messages, if any.
  *  Mailbox lock MUST BE HELD before calling this function.
+ *
+ *  Caches the message-id, references, and sender,
+ *  and passes the message to libbalsa_mailbox_cache_message for caching
+ *  other info.
  */
 void
 libbalsa_mailbox_local_cache_message(LibBalsaMailboxLocal * local,
@@ -907,6 +1017,9 @@ libbalsa_mailbox_local_cache_message(LibBalsaMailboxLocal * local,
     gpointer *entry;
     LibBalsaMailboxLocalInfo *info;
 
+    if (!message)
+        return;
+
     libbalsa_mailbox_cache_message(LIBBALSA_MAILBOX(local), msgno,
                                    message);
 
@@ -936,6 +1049,8 @@ libbalsa_mailbox_local_load_messages(LibBalsaMailbox *mailbox,
     LibBalsaMailboxLocal *local;
     guint lastno;
     GNode *lastn;
+    LibBalsaMailboxLocalMessageInfo *(*get_info) (LibBalsaMailboxLocal *,
+                                                  guint);
 
     g_return_if_fail(LIBBALSA_IS_MAILBOX_LOCAL(mailbox));
 
@@ -951,13 +1066,15 @@ libbalsa_mailbox_local_load_messages(LibBalsaMailbox *mailbox,
     lastno = libbalsa_mailbox_total_messages(mailbox);
     new_messages = lastno - msgno;
     lastn = g_node_last_child(mailbox->msg_tree);
+    get_info = LIBBALSA_MAILBOX_LOCAL_GET_CLASS(local)->get_info;
     while (++msgno <= lastno){
-	LibBalsaMessage *msg =
-	    libbalsa_mailbox_local_load_message(local, &lastn, msgno);
-	if (msg) {
-            libbalsa_mailbox_local_cache_message(local, msgno, msg);
-            g_object_unref(msg);
-        }
+        LibBalsaMailboxLocalMessageInfo *msg_info = get_info(local, msgno);
+
+	libbalsa_mailbox_local_load_message(local, &lastn, msgno, msg_info);
+
+	if (msg_info->message)
+            libbalsa_mailbox_local_cache_message(local, msgno,
+                                                 msg_info->message);
     }
 
     gdk_threads_leave();
@@ -1015,7 +1132,7 @@ libbalsa_mailbox_local_set_threading(LibBalsaMailbox * mailbox,
                 /* Get message info for all messages that weren't restored,
                  * so we can thread and sort them correctly before the
                  * mailbox is displayed. */
-                libbalsa_mailbox_prepare_threading(mailbox, NULL, total);
+                libbalsa_mailbox_prepare_threading(mailbox, total);
             libbalsa_mailbox_local_load_messages(mailbox, total);
         }
 
@@ -1070,6 +1187,7 @@ lbm_local_update_view_filter(LibBalsaMailbox * mailbox,
     LibBalsaProgress progress = LIBBALSA_PROGRESS_INIT;
     LibBalsaMailboxSearchIter *iter_view;
     guint msgno;
+    gboolean is_flag_only = TRUE;
 
     total = libbalsa_mailbox_total_messages(mailbox);
     if (view_filter
@@ -1079,6 +1197,7 @@ lbm_local_update_view_filter(LibBalsaMailbox * mailbox,
         text = g_strdup_printf(_("Filtering %s"), mailbox->name);
         libbalsa_progress_set_text(&progress, text, total);
         g_free(text);
+        is_flag_only = FALSE;
     }
 
     iter_view = libbalsa_mailbox_search_iter_new(view_filter);
@@ -1091,7 +1210,10 @@ lbm_local_update_view_filter(LibBalsaMailbox * mailbox,
     libbalsa_progress_set_text(&progress, NULL, 0);
     libbalsa_mailbox_search_iter_free(iter_view);
 
-    lbm_local_queue_save_tree(LIBBALSA_MAILBOX_LOCAL(mailbox));
+    /* If this is not a flags-only filter, the new mailbox tree is
+     * temporary, so we don't want to save it. */
+    if (is_flag_only)
+        lbm_local_queue_save_tree(LIBBALSA_MAILBOX_LOCAL(mailbox));
 }
 
 /*
@@ -1113,6 +1235,9 @@ lbm_local_prepare_msgno(LibBalsaMailboxLocal * local, guint msgno)
 
     message =
         libbalsa_mailbox_get_message((LibBalsaMailbox *) local, msgno);
+    if (!message)
+        return FALSE;
+
     libbalsa_mailbox_local_cache_message(local, msgno, message);
     g_object_unref(message);
 
@@ -1141,47 +1266,44 @@ lbm_local_thread_idle(LibBalsaMailboxLocal * local)
     return FALSE;
 }
 
-/* The class method; prepare messages in the list, or if msgnos == NULL,
- * from len + 1 to the end of the mailbox. */
-static void
+/* The class method; prepare messages from start + 1 to the end of the
+ * mailbox; return TRUE if successful. */
+static gboolean
 libbalsa_mailbox_local_prepare_threading(LibBalsaMailbox * mailbox,
-                                         guint * msgnos, guint len)
+                                         guint start)
 {
     LibBalsaMailboxLocal *local = LIBBALSA_MAILBOX_LOCAL(mailbox);
     guint msgno;
     gboolean need_thread = FALSE;
+    gchar *text;
+    guint total;
+    LibBalsaProgress progress = LIBBALSA_PROGRESS_INIT;
+    gboolean retval = TRUE;
 
     libbalsa_mailbox_local_set_threading_info(local);
 
-    if (msgnos) {
-        while (len) {
-            if (lbm_local_prepare_msgno(local, *msgnos++))
-                need_thread = TRUE;
-            --len;
-        }
-    } else {
-        gchar *text;
-        guint total;
-        LibBalsaProgress progress = LIBBALSA_PROGRESS_INIT;
+    text = g_strdup_printf(_("Preparing %s"), mailbox->name);
+    total = libbalsa_mailbox_total_messages(mailbox);
+    libbalsa_progress_set_text(&progress, text, total - start);
+    g_free(text);
 
-        text = g_strdup_printf(_("Preparing %s"), mailbox->name);
-        total = libbalsa_mailbox_total_messages(mailbox);
-        libbalsa_progress_set_text(&progress, text, total - len);
-        g_free(text);
-
-        for (msgno = len + 1; msgno <= total; msgno++) {
-            if (lbm_local_prepare_msgno(local, msgno)) {
-                need_thread = TRUE;
-                libbalsa_progress_set_fraction(&progress,
-                                               ((gdouble) msgno) /
-                                               ((gdouble) (total - len)));
+    for (msgno = start + 1; msgno <= total; msgno++) {
+        if (lbm_local_prepare_msgno(local, msgno)) {
+            need_thread = TRUE;
+            libbalsa_progress_set_fraction(&progress,
+                                           ((gdouble) msgno) /
+                                           ((gdouble) (total - start)));
+            if (!MAILBOX_OPEN(mailbox)) {
+                /* Mailbox was closed during set-fraction. */
+                retval = FALSE;
+                break;
             }
         }
-
-        libbalsa_progress_set_text(&progress, NULL, 0);
     }
 
-    if (need_thread && !local->thread_id) {
+    libbalsa_progress_set_text(&progress, NULL, 0);
+
+    if (retval && need_thread && !local->thread_id) {
         LibBalsaMailbox *mailbox = LIBBALSA_MAILBOX(local);
 
         if (libbalsa_mailbox_get_threading_type(mailbox) !=
@@ -1193,6 +1315,8 @@ libbalsa_mailbox_local_prepare_threading(LibBalsaMailbox * mailbox,
                 g_idle_add((GSourceFunc) lbm_local_thread_idle, local);
         }
     }
+
+    return retval;
 }
 
 /* fetch message structure method: all local mailboxes have their own
@@ -1273,6 +1397,7 @@ struct _ThreadingInfo {
     GHashTable *id_table;
     GHashTable *subject_table;
     GSList *unthreaded;
+    LibBalsaMailboxThreadingType type;
 };
 typedef struct _ThreadingInfo ThreadingInfo;
 
@@ -1579,10 +1704,7 @@ static const gchar *
 lbml_get_subject(GNode * node, ThreadingInfo * ti)
 {
     guint msgno = GPOINTER_TO_UINT(((GNode *) node->data)->data);
-    return msgno <= ti->mailbox->mindex->len
-        && g_ptr_array_index(ti->mailbox->mindex, msgno - 1)
-        ? libbalsa_mailbox_msgno_get_subject(ti->mailbox, msgno)
-        : NULL;
+    return libbalsa_mailbox_msgno_get_subject(ti->mailbox, msgno);
 }
 
 static void
@@ -1845,6 +1967,7 @@ lbml_threading_simple(LibBalsaMailbox * mailbox,
 	g_node_traverse(msg_tree, G_POST_ORDER, G_TRAVERSE_ALL,
 			-1, (GNodeTraverseFunc) lbml_insert_message, &ti);
 
+    ti.type = type;
     g_node_traverse(msg_tree, G_POST_ORDER, G_TRAVERSE_ALL, -1,
 		    (GNodeTraverseFunc) lbml_thread_message, &ti);
 
@@ -1876,26 +1999,33 @@ lbml_insert_message(GNode * node, ThreadingInfo * ti)
 static gboolean
 lbml_thread_message(GNode * node, ThreadingInfo * ti)
 {
-    LibBalsaMailboxLocalInfo *info;
-    GNode *parent = NULL;
-
     if (!node->parent)
         return FALSE;
 
-    info = lbml_get_info(node, ti);
+    if (ti->type == LB_MAILBOX_THREADING_FLAT) {
+        if (node->parent != ti->mailbox->msg_tree)
+            libbalsa_mailbox_unlink_and_prepend(ti->mailbox, node,
+                                                ti->mailbox->msg_tree);
+    } else {
+        LibBalsaMailboxLocalInfo *info;
+        GList *refs;
+        GNode *parent = NULL;
 
-    if (info) {
-        GList *refs = info->refs_for_threading;
+        info = lbml_get_info(node, ti);
+        if (!info)
+            return FALSE;
+
+        refs = info->refs_for_threading;
         if (refs)
             parent = g_hash_table_lookup(ti->id_table,
                                          g_list_last(refs)->data);
-    }
 
-    if (!parent)
-        parent = ti->mailbox->msg_tree;
-    if (parent != node->parent && parent != node
-        && !g_node_is_ancestor(node, parent))
-        libbalsa_mailbox_unlink_and_prepend(ti->mailbox, node, parent);
+        if (!parent)
+            parent = ti->mailbox->msg_tree;
+        if (parent != node->parent && parent != node
+            && !g_node_is_ancestor(node, parent))
+            libbalsa_mailbox_unlink_and_prepend(ti->mailbox, node, parent);
+    }
 
     return FALSE;
 }
@@ -1957,7 +2087,7 @@ libbalsa_mailbox_local_get_message_stream(LibBalsaMailbox * mailbox,
 /* Queued sync. */
 
 static void
-lbml_sync_real(LibBalsaMailboxLocal * local)
+lbm_local_sync_real(LibBalsaMailboxLocal * local)
 {
     LibBalsaMailbox *mailbox = (LibBalsaMailbox*)local;
     time_t tstart;
@@ -1978,28 +2108,28 @@ lbml_sync_real(LibBalsaMailboxLocal * local)
 }
 
 static gboolean
-lbml_sync_idle(LibBalsaMailboxLocal * local)
+lbm_local_sync_idle(LibBalsaMailboxLocal * local)
 {
 #ifdef BALSA_USE_THREADS
     pthread_t sync_thread;
 
-    pthread_create(&sync_thread, NULL, (void *) lbml_sync_real, local);
+    pthread_create(&sync_thread, NULL, (void *) lbm_local_sync_real, local);
     pthread_detach(sync_thread);
 #else                           /*BALSA_USE_THREADS */
-    lbml_sync_real(local);
+    lbm_local_sync_real(local);
 #endif                          /*BALSA_USE_THREADS */
 
     return FALSE;
 }
 
-void
-libbalsa_mailbox_local_queue_sync(LibBalsaMailboxLocal * local)
+static void
+lbm_local_sync_queue(LibBalsaMailboxLocal * local)
 {
     guint schedule_delay;
 
     /* The optimal behavior here would be to keep rescheduling
     * requests.  But think of following: the idle handler started and
-    * triggered lbml_sync_real thread. While it waits for the lock,
+    * triggered lbm_local_sync_real thread. While it waits for the lock,
     * another queue request is filed but it is too late for removal of
     * the sync thread. And we get two sync threads executing one after
     * another, etc. So it is better to do sync bit too often... */
@@ -2011,15 +2141,88 @@ libbalsa_mailbox_local_queue_sync(LibBalsaMailboxLocal * local)
      * annnoyed. */
     schedule_delay = (local->sync_time*5000)/local->sync_cnt;
     local->sync_id = g_timeout_add_full(G_PRIORITY_LOW, schedule_delay,
-                                        (GSourceFunc)lbml_sync_idle,
+                                        (GSourceFunc)lbm_local_sync_idle,
                                         local, NULL);
 }
 
 static void
 lbm_local_sort(LibBalsaMailbox * mailbox, GArray *sort_array)
 {
-    lbm_local_queue_save_tree(LIBBALSA_MAILBOX_LOCAL(mailbox));
     LIBBALSA_MAILBOX_CLASS(parent_class)->sort(mailbox, sort_array);
+    lbm_local_queue_save_tree(LIBBALSA_MAILBOX_LOCAL(mailbox));
+}
+
+#define FLAGS_REALLY_DIFFER(flags0, flags1) \
+        (((flags0 ^ flags1) & LIBBALSA_MESSAGE_FLAGS_REAL) != 0)
+
+static gboolean
+libbalsa_mailbox_local_messages_change_flags(LibBalsaMailbox * mailbox,
+                                             GArray * msgnos,
+                                             LibBalsaMessageFlag set,
+                                             LibBalsaMessageFlag clear)
+{
+    LibBalsaMailboxLocal *local = LIBBALSA_MAILBOX_LOCAL(mailbox);
+    LibBalsaMailboxLocalMessageInfo *(*get_info) (LibBalsaMailboxLocal *,
+                                                  guint) =
+        LIBBALSA_MAILBOX_LOCAL_GET_CLASS(local)->get_info;
+    guint i;
+    guint changed = 0;
+
+    for (i = 0; i < msgnos->len; i++) {
+        guint msgno = g_array_index(msgnos, guint, i);
+        LibBalsaMailboxLocalMessageInfo *msg_info;
+        LibBalsaMessageFlag old_flags;
+        gboolean was_unread_undeleted, is_unread_undeleted;
+
+        if (!(msgno > 0
+              && msgno <= libbalsa_mailbox_total_messages(mailbox))) {
+            g_warning("msgno %u out of range", msgno);
+            continue;
+        }
+
+        msg_info = get_info(local, msgno);
+        old_flags = msg_info->flags;
+        msg_info->flags |= set;
+        msg_info->flags &= ~clear;
+        if (!FLAGS_REALLY_DIFFER(msg_info->flags, old_flags))
+            /* No real flags changed. */
+            continue;
+        ++changed;
+
+        if (msg_info->message)
+            msg_info->message->flags = msg_info->flags;
+
+        libbalsa_mailbox_index_set_flags(mailbox, msgno, msg_info->flags);
+
+        was_unread_undeleted = (old_flags & LIBBALSA_MESSAGE_FLAG_NEW)
+            && !(old_flags & LIBBALSA_MESSAGE_FLAG_DELETED);
+        is_unread_undeleted = (msg_info->flags & LIBBALSA_MESSAGE_FLAG_NEW)
+            && !(msg_info->flags & LIBBALSA_MESSAGE_FLAG_DELETED);
+        mailbox->unread_messages +=
+            is_unread_undeleted - was_unread_undeleted;
+    }
+
+    if (changed > 0) {
+        libbalsa_mailbox_set_unread_messages_flag(mailbox,
+                                                  mailbox->unread_messages
+                                                  > 0);
+        lbm_local_sync_queue(local);
+    }
+
+    return TRUE;
+}
+
+static gboolean
+libbalsa_mailbox_local_msgno_has_flags(LibBalsaMailbox * mailbox,
+                                       guint msgno,
+                                       LibBalsaMessageFlag set,
+                                       LibBalsaMessageFlag unset)
+{
+    LibBalsaMailboxLocal *local = LIBBALSA_MAILBOX_LOCAL(mailbox);
+    LibBalsaMailboxLocalMessageInfo *msg_info =
+        LIBBALSA_MAILBOX_LOCAL_GET_CLASS(local)->get_info(local, msgno);
+
+    return (msg_info->flags & set) == set && (msg_info->flags & unset) == 0;
 }
 
 static GArray *
@@ -2033,12 +2236,13 @@ libbalsa_mailbox_local_duplicate_msgnos(LibBalsaMailbox * mailbox)
     if (!local->threading_info)
         return NULL;
 
+    /* We need all the message-ids. */
+    if (!libbalsa_mailbox_prepare_threading(mailbox, 0))
+        return NULL;
+
     table = g_hash_table_new(g_str_hash, g_str_equal);
     msgnos = g_array_new(FALSE, FALSE, sizeof(guint));
 
-    /* We need all the message-ids. */
-    libbalsa_mailbox_prepare_threading(mailbox, NULL, 0);
-
     for (i = 0; i < local->threading_info->len; i++) {
         LibBalsaMailboxLocalInfo *info;
         gpointer tmp;
@@ -2071,6 +2275,7 @@ libbalsa_mailbox_local_duplicate_msgnos(LibBalsaMailbox * mailbox)
     return msgnos;
 }
 
+/* LibBalsaMailboxLocal class method: */
 static void
 lbm_local_real_remove_files(LibBalsaMailboxLocal * local)
 {
