@@ -37,10 +37,11 @@
 #include "libbalsa.h"
 #include "libbalsa-conf.h"
 #include "pop3.h"
+#include "server.h"
 #include "misc.h"
 #include "mailbox.h"
 #include "mailbox_pop3.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 
 int PopDebug = 0;
 
