@@ -47,7 +47,7 @@
 #include "libbalsa_private.h"
 #include "misc.h"
 #include "html.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 
 static const gchar *libbalsa_get_codeset_name(const gchar *txt, 
 					      LibBalsaCodeset Codeset);
@@ -518,8 +518,19 @@ libbalsa_mktempdir (char **s)
 static LibBalsaCodeset sanitize_fallback_codeset = WEST_EUROPE;
 LibBalsaCodeset
 libbalsa_set_fallback_codeset(LibBalsaCodeset codeset)
-{     
+{
     LibBalsaCodeset ret = sanitize_fallback_codeset;
+#if defined(HAVE_GMIME_2_2_7)
+    const gchar *charsets[] = {
+        g_strdup("UTF-8"),
+        g_strdup(libbalsa_get_codeset_name(NULL, codeset)),
+        NULL
+    };
+
+    g_mime_set_user_charsets(charsets);
+    /* GMime will free the strings. */
+#endif                          /* HAVE_GMIME_2_2_7 */
+
     sanitize_fallback_codeset = codeset;
     return ret;
 }
