@@ -1,4 +1,4 @@
-/* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
+/* -*-mode:c; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
  * Copyright (C) 1997-2002 Stuart Parmenter and others,
@@ -39,6 +39,7 @@
 #include "libbalsa.h"
 #include "libbalsa_private.h"
 
+#include "server.h"
 #include "send.h"
 #include "misc.h"
 #include "information.h"
@@ -53,7 +54,7 @@
 
 #include <sys/utsname.h>
 
-#include "i18n.h"
+#include <glib/gi18n.h>
 
 typedef struct _MessageQueueItem MessageQueueItem;
 
@@ -71,6 +72,7 @@ struct _MessageQueueItem {
     long acc;
     long update;
     int refcount;
+    gboolean error;
 #endif
 };
 
@@ -136,6 +138,8 @@ msg_queue_item_new(LibBalsaFccboxFinder finder)
 #if !ENABLE_ESMTP
     mqi->next_message = NULL;
     mqi->status = MQI_WAITING;
+#else
+    mqi->error = FALSE;
 #endif
     mqi->stream=NULL;
     mqi->tempfile[0] = '\0';
@@ -539,8 +543,8 @@ libbalsa_message_cb (void **buf, int *len, void *arg)
 }
 
 static void
-add_recipients(smtp_message_t message, smtp_message_t bcc_message, 
-               InternetAddressList *recipient_list, const char *header)
+add_recipients(smtp_message_t message,
+               InternetAddressList * recipient_list)
 {
     for (; recipient_list; recipient_list = recipient_list->next) {
         InternetAddress *ia = recipient_list->address;
@@ -548,7 +552,7 @@ add_recipients(smtp_message_t message, smtp_message_t bcc_message,
 	if (ia->type == INTERNET_ADDRESS_NAME)
 	    smtp_add_recipient (message, ia->value.addr);
 	else if (ia->type == INTERNET_ADDRESS_GROUP)
-	    add_recipients(message, bcc_message, ia->value.members, header);
+	    add_recipients(message, ia->value.members);
 
             /* XXX  - this is where to add DSN requests.  It would be
                cool if LibBalsaAddress could contain DSN options
@@ -556,6 +560,13 @@ add_recipients(smtp_message_t message, smtp_message_t bcc_message,
     }
 }
 
+static gboolean
+lbs_list_has_one_address(InternetAddressList * recipient_list)
+{
+    return recipient_list && !recipient_list->next && 
+	recipient_list->address->type == INTERNET_ADDRESS_NAME;
+}
+
 /* libbalsa_process_queue:
    treats given mailbox as a set of messages to send. Loads them up and
    launches sending thread/routine.
@@ -620,13 +631,16 @@ lbs_process_queue(LibBalsaMailbox * outbox, LibBalsaFccboxFinder finder,
 	const gchar *smtp_server_name;
         LibBalsaMsgCreateResult created;
 
-        if (libbalsa_mailbox_msgno_has_flags(outbox, msgno, 
-                                             (LIBBALSA_MESSAGE_FLAG_FLAGGED |
-                                              LIBBALSA_MESSAGE_FLAG_DELETED),
-                                             0))
+        /* Skip this message if it either FLAGGED or DELETED: */
+        if (!libbalsa_mailbox_msgno_has_flags
+            (outbox, msgno, 0,
+             (LIBBALSA_MESSAGE_FLAG_FLAGGED |
+              LIBBALSA_MESSAGE_FLAG_DELETED)))
             continue;
 
 	msg = libbalsa_mailbox_get_message(outbox, msgno);
+        if (!msg) /* error? */
+            continue;
         libbalsa_message_body_ref(msg, TRUE, TRUE);
         smtp_server_name =
             libbalsa_message_get_user_header(msg, "X-Balsa-SmtpServer");
@@ -686,22 +700,20 @@ lbs_process_queue(LibBalsaMailbox * outbox, LibBalsaFccboxFinder finder,
 		new_message->stream = mem_stream;
 	    }
 
-	    /* If the Bcc: recipient list is present, add a additional
-	       copy of the message to the session.  The recipient list
-	       for the main copy of the message is generated from the
-	       To: and Cc: recipient list and libESMTP is asked to strip
-	       the Bcc: header.  The BCC copy of the message recipient
-	       list is taken from the Bcc recipient list and the Bcc:
-	       header is preserved in the message. */
-	    if (!msg->headers->bcc_list)
-		bcc_message = NULL;
-	    else
+            /* If the Bcc: recipient list is present and contains
+             * exactly one address, add a additional copy of the message
+             * to the session for that one recipient, in which the Bcc:
+             * header is preserved.  The main copy never contains a Bcc:
+             * header. */
+            if (lbs_list_has_one_address(msg->headers->bcc_list))
 		bcc_message = smtp_add_message (session);
+	    else
+		bcc_message = NULL;
             new_message->refcount = bcc_message ? 2 : 1;
 
 	    /* Add this after the Bcc: copy. */
 	    message = smtp_add_message (session);
-	    if (bcc_message)
+	    if (msg->headers->bcc_list)
 		smtp_set_header_option (message, "Bcc", Hdr_PROHIBIT, 1);
 
 	    smtp_message_set_application_data (message, new_message);
@@ -776,21 +788,21 @@ lbs_process_queue(LibBalsaMailbox * outbox, LibBalsaFccboxFinder finder,
 #endif
 
 	    /* Now need to add the recipients to the message.  The main
-	       copy of the message gets the To and Cc recipient list.
-	       The bcc copy gets the Bcc recipients.  */
+	       copy of the message gets the To and Cc recipient list, and
+	       the Bcc recipient list, when it has more than one address.
+	       The bcc copy gets the single Bcc recipient.  */
 
-            add_recipients(message, bcc_message, msg->headers->to_list, "To");
-            add_recipients(message, bcc_message, msg->headers->cc_list, "Cc");
+            add_recipients(message, msg->headers->to_list);
+            add_recipients(message, msg->headers->cc_list);
 
-            add_recipients(bcc_message, NULL, msg->headers->bcc_list, "Cc");
+            add_recipients(bcc_message ? bcc_message : message, 
+                           msg->headers->bcc_list);
 
 	    /* Prohibit status headers. */
 	    smtp_set_header_option(message, "Status", Hdr_PROHIBIT, 1);
 	    smtp_set_header_option(message, "X-Status", Hdr_PROHIBIT, 1);
-            /* ... and FCC header. */
-            smtp_set_header_option(message, "X-Balsa-Fcc", Hdr_PROHIBIT, 1);
-            smtp_set_header_option(message, "X-Balsa-SmtpServer",
-		                   Hdr_PROHIBIT, 1);
+            /* ... and all X-Balsa-* headers. */
+            smtp_set_header_option(message, "X-Balsa-", Hdr_PROHIBIT, 1);
 
 
 	    /* Estimate the size of the message.  This need not be exact
@@ -891,7 +903,7 @@ handle_successful_send(smtp_message_t message, void *be_verbose)
     if (mqi != NULL)
       mqi->refcount--;
 
-    if(mqi != NULL && mqi->orig != NULL && mqi->orig->mailbox)
+    if(mqi != NULL && mqi->orig != NULL && mqi->orig->mailbox && !mqi->error)
 	libbalsa_message_change_flags(mqi->orig, 0,
                                       LIBBALSA_MESSAGE_FLAG_FLAGGED);
     else printf("mqi: %p mqi->orig: %p mqi->orig->mailbox: %p\n",
@@ -900,7 +912,7 @@ handle_successful_send(smtp_message_t message, void *be_verbose)
     status = smtp_message_transfer_status (message);
     if (status->code / 100 == 2) {
 	if (mqi != NULL && mqi->orig != NULL && mqi->refcount <= 0 &&
-            mqi->orig->mailbox) {
+            mqi->orig->mailbox && !mqi->error) {
             gboolean remove = TRUE;
             const gchar *fccurl =
                 libbalsa_message_get_user_header(mqi->orig, "X-Balsa-Fcc");
@@ -929,12 +941,22 @@ handle_successful_send(smtp_message_t message, void *be_verbose)
                                           LIBBALSA_MESSAGE_FLAG_FLAGGED, 0);
 	}
     } else {
+        /* Record the error, so that the message will not be deleted
+         * later... */
+        mqi->error = TRUE;
+        /* ...and mark it as:
+         *   - flagged, so it will not be sent again until the error
+         *     is fixed and the user manually clears the flag;
+         *   - undeleted, in case it was already deleted. */
+        libbalsa_message_change_flags(mqi->orig,
+                                      LIBBALSA_MESSAGE_FLAG_FLAGGED,
+                                      LIBBALSA_MESSAGE_FLAG_DELETED);
 	/* Check whether it was a problem with transfer. */
         if(*(gboolean*)be_verbose) {
             int cnt = 0;
             if(status->code != 250 && status->code != 0) {
                 libbalsa_information(
-                                     LIBBALSA_INFORMATION_WARNING,
+                                     LIBBALSA_INFORMATION_ERROR,
                                      _("Relaying refused:\n"
                                        "%d: %s\n"
                                        "Message left in your outbox.\n"), 
@@ -946,7 +968,7 @@ handle_successful_send(smtp_message_t message, void *be_verbose)
                 status = smtp_reverse_path_status(message);
                 if(status->code != 250 && status->code != 0) {
                     libbalsa_information
-                        (LIBBALSA_INFORMATION_WARNING,
+                        (LIBBALSA_INFORMATION_ERROR,
                          _("Relaying refused:\n"
                            "%d: %s\n"
                            "Message left in your outbox.\n"), 
@@ -1000,7 +1022,7 @@ libbalsa_smtp_event_cb (smtp_session_t session, int event_no, void *arg, ...)
         snprintf (buf, sizeof buf, _("From %s: %d %s"), 
 	          mailbox, status->code, status->text);
         g_strchomp(buf);
-	libbalsa_information(LIBBALSA_INFORMATION_DEBUG, buf);
+	libbalsa_information(LIBBALSA_INFORMATION_DEBUG, "%s", buf);
         break;
     case SMTP_EV_RCPTSTATUS:
         mailbox = va_arg (ap, const char *);
@@ -1015,7 +1037,7 @@ libbalsa_smtp_event_cb (smtp_session_t session, int event_no, void *arg, ...)
         snprintf (buf, sizeof buf, _("To %s: %d %s"),
 	          mailbox, status->code, status->text);
         g_strchomp(buf);
-	libbalsa_information(LIBBALSA_INFORMATION_DEBUG, buf);
+	libbalsa_information(LIBBALSA_INFORMATION_DEBUG, "%s", buf);
         break;
     case SMTP_EV_MESSAGEDATA:
         message = va_arg (ap, smtp_message_t);
@@ -1040,7 +1062,7 @@ libbalsa_smtp_event_cb (smtp_session_t session, int event_no, void *arg, ...)
 	snprintf (buf, sizeof buf, "%d %s", status->code, status->text);
         g_strchomp(buf);
 	MSGSENDTHREAD(threadmsg, MSGSENDTHREADPROGRESS, buf, NULL, NULL, 0);
-	libbalsa_information(LIBBALSA_INFORMATION_DEBUG, buf);
+	libbalsa_information(LIBBALSA_INFORMATION_DEBUG, "%s", buf);
         /* Reset 'mqi->sent' for the next message (i.e. bcc copy) */
         mqi = smtp_message_get_application_data (message);
         if (mqi != NULL) {
@@ -1182,6 +1204,8 @@ libbalsa_process_queue(LibBalsaMailbox* outbox, LibBalsaFccboxFinder finder,
             continue;
 
 	msg = libbalsa_mailbox_get_message(outbox, msgno);
+        if (!msg) /* error? */
+            continue;
         libbalsa_message_body_ref(msg, TRUE, TRUE); /* FIXME: do we need
                                                       * all headers? */
 	new_message = msg_queue_item_new(finder);
@@ -1281,6 +1305,10 @@ monitor_cb (const char *buf, int buflen, int writing, void *arg)
     }
 
  fputs (writing ? "C: " : "S: ", fp);
+ if (writing && g_ascii_strncasecmp(buf, "auth plain", 10) == 0) {
+     fputs("AUTH (details hidden)\n", fp);
+     return;
+ }
  if (fwrite (buf, 1, buflen, fp) != (size_t) buflen)
    /* FIXME */ return;
  if (buf[buflen - 1] != '\n')
