@@ -47,14 +47,16 @@ LibBalsaMsgCreateResult libbalsa_message_queue(LibBalsaMessage* message,
                                                LibBalsaMailbox* fccbox,
                                                LibBalsaSmtpServer *
                                                smtp_server,
-					       gboolean flow);
+					       gboolean flow,
+					       GError ** error);
 LibBalsaMsgCreateResult libbalsa_message_send(LibBalsaMessage * message,
                                               LibBalsaMailbox * outbox,
                                               LibBalsaMailbox * fccbox,
                                               LibBalsaFccboxFinder finder,
                                               LibBalsaSmtpServer *
                                               smtp_server, gboolean flow,
-                                              gboolean debug);
+                                              gboolean debug,
+					      GError ** error);
 gboolean libbalsa_process_queue(LibBalsaMailbox * outbox,
                                 LibBalsaFccboxFinder finder,
                                 GSList * smtp_servers, gboolean debug);
@@ -63,12 +65,14 @@ gboolean libbalsa_process_queue(LibBalsaMailbox * outbox,
 LibBalsaMsgCreateResult libbalsa_message_queue(LibBalsaMessage* message, 
 					       LibBalsaMailbox* outbox,
                                                LibBalsaMailbox* fccbox,
-					       gboolean flow);
+					       gboolean flow,
+					       GError ** error);
 LibBalsaMsgCreateResult libbalsa_message_send(LibBalsaMessage* message,
 					      LibBalsaMailbox* outbox,  
 					      LibBalsaMailbox* fccbox,
                                               LibBalsaFccboxFinder finder, 
-					      gboolean flow, gboolean debug);
+					      gboolean flow, gboolean debug,
+					      GError ** error);
 gboolean libbalsa_process_queue(LibBalsaMailbox* outbox, 
                                 LibBalsaFccboxFinder finder,
                                 gboolean debug);
@@ -95,7 +99,9 @@ typedef struct {
   t_message->msg = s_msg; \
   t_message->mbox = s_mbox; \
   t_message->of_total = messof; \
-  write( send_thread_pipes[1], (void *) &t_message, sizeof(void *) );
+  if (write( send_thread_pipes[1], (void *) &t_message, sizeof(void *) ) \
+      < (ssize_t) sizeof(void *)) \
+    g_warning("pipe error");
 
 enum {
     MSGSENDTHREADERROR,
