@@ -30,8 +30,7 @@
 #include "libbalsa.h"
 #include "libbalsa_private.h"
 #include "misc.h"
-#include "mime-stream-shared.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 
 static void close_cb(GtkAction * action, gpointer data);
 static void copy_cb(GtkAction * action, gpointer data);
@@ -41,9 +40,9 @@ static void lsv_escape_cb(GtkAction * action, gpointer data);
 /* Normal items */
 static GtkActionEntry entries[] = {
     /* Top level */
-    {"FileMenu", NULL, "_File"},
-    {"EditMenu", NULL, "_Edit"},
-    {"ViewMenu", NULL, "_View"},
+    {"FileMenu", NULL, N_("_File")},
+    {"EditMenu", NULL, N_("_Edit")},
+    {"ViewMenu", NULL, N_("_View")},
     /* Items */
     {"Close", GTK_STOCK_CLOSE, N_("_Close"), "<control>W",
      N_("Close the window"), G_CALLBACK(close_cb)},
@@ -160,9 +159,9 @@ lsv_escape_cb(GtkAction * action, gpointer data)
 	return;
 
     mem_stream = g_mime_stream_mem_new();
-    libbalsa_mime_stream_shared_lock(msg_stream);
+    libbalsa_mailbox_lock_store(msg->mailbox);
     g_mime_stream_write_to_stream(msg_stream, mem_stream);
-    libbalsa_mime_stream_shared_unlock(msg_stream);
+    libbalsa_mailbox_unlock_store(msg->mailbox);
     g_mime_stream_write(mem_stream, "", 1); /* close string */
     raw_message = (char *) GMIME_STREAM_MEM(mem_stream)->buffer->data;
 
@@ -198,6 +197,8 @@ lbsv_app_set_menus(GtkWindow * app, GtkAction ** action)
     window = GTK_WIDGET(app);
 
     action_group = gtk_action_group_new("MenuActions");
+    gtk_action_group_set_translation_domain(action_group, NULL);
+
     gtk_action_group_add_actions(action_group, entries,
                                  G_N_ELEMENTS(entries), window);
     gtk_action_group_add_toggle_actions(action_group, toggle_entries,
