@@ -41,10 +41,13 @@
 #if HAVE_SQLITE
 #include "address-book-gpe.h"
 #endif /* HAVE_SQLITE */
+#if HAVE_RUBRICA
+#include "address-book-rubrica.h"
+#endif /* HAVE_RUBRICA */
 #include "address-book-config.h"
 #include "libbalsa-conf.h"
 #include "libbalsa.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 
 struct ABMainWindow {
     GtkWindow *window;
@@ -198,6 +201,10 @@ bab_window_set_title(LibBalsaAddressBook * address_book)
     else if (LIBBALSA_IS_ADDRESS_BOOK_GPE(address_book))
         type = "GPE";
 #endif
+#if HAVE_RUBRICA
+    else if (LIBBALSA_IS_ADDRESS_BOOK_RUBRICA(address_book))
+        type = "Rubrica";
+#endif /* HAVE_RUBRICA */
 
     title =
         g_strconcat(type, _(" address book: "), address_book->name, NULL);
@@ -384,6 +391,16 @@ file_new_gpe_cb(GtkAction * action, gpointer user_data)
 }
 #endif /* HAVE_SQLITE */
 
+#if HAVE_RUBRICA
+static void
+file_new_rubrica_cb(GtkAction * action, gpointer user_data)
+{
+    balsa_address_book_config_new_from_type
+        (LIBBALSA_TYPE_ADDRESS_BOOK_RUBRICA, address_book_change,
+         contacts_app.window);
+}
+#endif /* HAVE_RUBRICA */
+
 static void
 file_properties_cb(GtkAction * action, gpointer user_data)
 {
@@ -467,10 +484,10 @@ edit_new_entry_cb(GtkAction * action, gpointer user_data)
 
 /* Normal items */
 static GtkActionEntry entries[] = {
-    {"FileMenu", NULL, "_File"},
-    {"EntryMenu", NULL, "_Entry"},
-    {"HelpMenu", NULL, "_Help"},
-    {"New", GTK_STOCK_NEW, "_New"},
+    {"FileMenu", NULL, N_("_File")},
+    {"EntryMenu", NULL, N_("_Entry")},
+    {"HelpMenu", NULL, N_("_Help")},
+    {"New", GTK_STOCK_NEW, N_("_New")},
     {"NewVcard", NULL, N_("VCard Address Book (GnomeCard)"), NULL, NULL,
      G_CALLBACK(file_new_vcard_cb)},
     {"NewExtern", NULL, N_("External query (a program)"), NULL, NULL,
@@ -485,21 +502,25 @@ static GtkActionEntry entries[] = {
     {"NewGpe", NULL, N_("GPE Address Book"), NULL, NULL,
      G_CALLBACK(file_new_gpe_cb)},
 #endif /* HAVE_SQLITE */
-    {"Properties", GTK_STOCK_PROPERTIES, "_Properties", NULL,
-     "Edit address book properties", G_CALLBACK(file_properties_cb)},
-    {"Delete", GTK_STOCK_DELETE, "_Delete", NULL,
-     "Delete address book", G_CALLBACK(file_delete_cb)},
-    {"Quit", GTK_STOCK_QUIT, "_Quit", NULL, "Exit the program",
+#if HAVE_RUBRICA
+    {"NewRubrica", NULL, N_("Rubrica Address Book"), NULL, NULL,
+     G_CALLBACK(file_new_rubrica_cb)},
+#endif /* HAVE_RUBRICA */
+    {"Properties", GTK_STOCK_PROPERTIES, N_("_Properties"), NULL,
+     N_("Edit address book properties"), G_CALLBACK(file_properties_cb)},
+    {"Delete", GTK_STOCK_DELETE, N_("_Delete"), NULL,
+     N_("Delete address book"), G_CALLBACK(file_delete_cb)},
+    {"Quit", GTK_STOCK_QUIT, N_("_Quit"), NULL, N_("Exit the program"),
      gtk_main_quit},
-    {"NewEntry", GTK_STOCK_NEW, "_New Entry", "<shift><control>N",
-     "Add new entry", G_CALLBACK(edit_new_entry_cb)},
+    {"NewEntry", GTK_STOCK_NEW, N_("_New Entry"), "<shift><control>N",
+     N_("Add new entry"), G_CALLBACK(edit_new_entry_cb)},
     {"About",
 #if GTK_CHECK_VERSION(2, 6, 0)
      GTK_STOCK_ABOUT,
 #else
      GNOME_STOCK_ABOUT,
 #endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
-     "_About", NULL, NULL, NULL}
+     N_("_About"), NULL, NULL, NULL}
 };
 
 static const char *ui_description =
@@ -516,6 +537,9 @@ static const char *ui_description =
 #if HAVE_SQLITE
 "        <menuitem action='NewGpe'/>"
 #endif /* HAVE_SQLITE */
+#if HAVE_RUBRICA
+"        <menuitem action='NewRubrica'/>"
+#endif /* HAVE_RUBRICA */
 "      </menu>"
 "      <menuitem action='Properties'/>"
 "      <menuitem action='Delete'/>"
@@ -544,6 +568,7 @@ get_main_menu(GtkWidget * window, GtkWidget ** menubar,
 
     contacts_app.action_group = action_group =
         gtk_action_group_new("MenuActions");
+    gtk_action_group_set_translation_domain(action_group, NULL);
     gtk_action_group_add_actions(action_group, entries,
                                  G_N_ELEMENTS(entries), window);
 
@@ -894,14 +919,19 @@ static GtkWidget*
 bab_window_new()
 {
     GtkWidget* menubar = NULL, *main_vbox, *cont_box, *vbox, *scroll;
-    GtkWidget *wnd = gnome_app_new("Contacts", "Contacts");
+    GtkWidget *wnd = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     GtkWidget *edit_box;
 
+    gtk_window_set_title(GTK_WINDOW(wnd), "Contacts");
+
+    /* main vbox */
+    main_vbox = gtk_vbox_new(FALSE, 1);
+    gtk_container_add(GTK_CONTAINER(wnd), main_vbox);
+
     get_main_menu(GTK_WIDGET(wnd), &menubar, contacts_app.address_book_list);
     if (menubar)
-	gnome_app_set_menus(GNOME_APP(wnd), GTK_MENU_BAR(menubar));
-    /* main vbox */
-    main_vbox = gtk_vbox_new(FALSE, 1);
+        gtk_box_pack_start(GTK_BOX(main_vbox),
+                           menubar, FALSE, FALSE, 1);
 
     /* Entry widget for finding an address */
     gtk_box_pack_start(GTK_BOX(main_vbox),
@@ -939,8 +969,8 @@ bab_window_new()
     g_signal_connect(wnd, "key-press-event",
 		     G_CALLBACK(ew_key_pressed), &contacts_app);
     gtk_window_set_default_size(GTK_WINDOW(wnd), 500, 400);
-    gnome_app_set_contents(GNOME_APP(wnd), main_vbox);
 
+    gtk_widget_show_all(wnd);
     return wnd;
 }
 
@@ -979,6 +1009,9 @@ bab_init(void)
 #if HAVE_SQLITE
     LIBBALSA_TYPE_ADDRESS_BOOK_GPE;
 #endif
+#if HAVE_RUBRICA
+    LIBBALSA_TYPE_ADDRESS_BOOK_RUBRICA;
+#endif /* HAVE_RUBRICA */
     memset(&contacts_app, 0, sizeof(contacts_app));
 }
 
@@ -1033,6 +1066,9 @@ main(int argc, char *argv[])
 #if HAVE_SQLITE
     LIBBALSA_TYPE_ADDRESS_BOOK_GPE;
 #endif
+#if HAVE_RUBRICA
+    LIBBALSA_TYPE_ADDRESS_BOOK_RUBRICA;
+#endif /* HAVE_RUBRICA */
     libbalsa_real_information_func = (LibBalsaInformationFunc)information_real;
     g_mime_init(0);
 
