@@ -29,7 +29,7 @@
 
 #include "balsa-app.h"
 #include "address-book-config.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 
 typedef struct _AddressBookConfig AddressBookConfig;
 struct _AddressBookConfig {
@@ -74,6 +74,9 @@ static GtkWidget *create_ldap_dialog(AddressBookConfig * abc);
 #ifdef HAVE_SQLITE
 static GtkWidget *create_gpe_dialog(AddressBookConfig * abc);
 #endif
+#ifdef HAVE_RUBRICA
+static GtkWidget *create_rubrica_dialog(AddressBookConfig * abc);
+#endif
 
 static void help_button_cb(AddressBookConfig * abc);
 static gboolean handle_close(AddressBookConfig * abc);
@@ -294,6 +297,10 @@ create_dialog_from_type(AddressBookConfig * abc)
 #ifdef HAVE_SQLITE
     } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_GPE) {
         return create_gpe_dialog(abc);
+#endif
+#ifdef HAVE_RUBRICA
+    } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_RUBRICA) {
+        return create_rubrica_dialog(abc);
 #endif
     } else {
         g_assert_not_reached();
@@ -431,7 +438,7 @@ create_ldap_dialog(AddressBookConfig * abc)
 
     LibBalsaAddressBookLdap* ab;
     GtkWidget* label;
-    gchar *host = libbalsa_guess_ldap_server();
+    gchar *host = g_strdup_printf("ldap://%s", libbalsa_guess_ldap_server());
     gchar *base = libbalsa_guess_ldap_base();
     gchar *name = libbalsa_guess_ldap_name();
 
@@ -517,6 +524,14 @@ create_gpe_dialog(AddressBookConfig * abc)
 }
 #endif
 
+#if HAVE_RUBRICA
+static GtkWidget *
+create_rubrica_dialog(AddressBookConfig * abc)
+{
+    return create_local_dialog(abc);
+}
+#endif
+
 static void
 help_button_cb(AddressBookConfig * abc)
 {
@@ -715,6 +730,14 @@ create_book(AddressBookConfig * abc)
     } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_GPE) {
         address_book =
             libbalsa_address_book_gpe_new(name);
+#endif
+#ifdef HAVE_RUBRICA
+    } else if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_RUBRICA) {
+        gchar *path =
+            gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(abc->window));
+        if (path != NULL)
+            address_book = libbalsa_address_book_rubrica_new(name, path);
+        g_free(path);
 #endif
     } else
         g_assert_not_reached();
@@ -740,7 +763,11 @@ modify_book(AddressBookConfig * abc)
         g_strdup(gtk_entry_get_text(GTK_ENTRY(abc->name_entry)));
 
     if (abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_VCARD
-        || abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_LDIF) {
+        || abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_LDIF
+#ifdef HAVE_RUBRICA
+        || abc->type == LIBBALSA_TYPE_ADDRESS_BOOK_RUBRICA
+#endif /* HAVE_RUBRICA */
+        ) {
         LibBalsaAddressBookText *ab_text =
             LIBBALSA_ADDRESS_BOOK_TEXT(address_book);
         gchar *path =
@@ -877,6 +904,18 @@ add_gpe_cb(GtkWidget * widget, AddressBookConfig * abc)
 }
 #endif /* HAVE_SQLITE */
 
+#ifdef HAVE_RUBRICA
+static void
+add_rubrica_cb(GtkWidget * widget, AddressBookConfig * abc)
+{
+    g_object_weak_unref(G_OBJECT(gtk_widget_get_parent(widget)),
+                        (GWeakNotify) g_free, abc);
+    abc->type = LIBBALSA_TYPE_ADDRESS_BOOK_RUBRICA;
+    abc->window = create_rubrica_dialog(abc);
+    gtk_widget_show_all(abc->window);
+}
+#endif /* HAVE_SQLITE */
+
 GtkWidget *
 balsa_address_book_add_menu(BalsaAddressBookCallback callback,
                             GtkWindow * parent)
@@ -922,5 +961,12 @@ balsa_address_book_add_menu(BalsaAddressBookCallback callback,
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 #endif /* HAVE_SQLITE */
 
+#ifdef HAVE_RUBRICA
+    menuitem = gtk_menu_item_new_with_label(_("Rubrica2 Address Book"));
+    g_signal_connect(G_OBJECT(menuitem), "activate",
+                     G_CALLBACK(add_rubrica_cb), abc);
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+#endif /* HAVE_RUBRICA */
+
     return menu;
 }
