@@ -34,11 +34,12 @@
 
 #include "filter-funcs.h"
 #include "misc.h"
+#include "server.h"
 #include "smtp-server.h"
 #include "balsa-app.h"
 #include "save-restore.h"
 
-#include "i18n.h"	/* Must come after balsa-app.h. */
+#include <glib/gi18n.h>	/* Must come after balsa-app.h. */
 
 /* Global application structure */
 struct BalsaApplication balsa_app;
@@ -295,8 +296,12 @@ balsa_app_init(void)
     balsa_app.mblist_width = 100;
     balsa_app.mw_width = MW_DEFAULT_WIDTH;
     balsa_app.mw_height = MW_DEFAULT_HEIGHT;
+    balsa_app.mw_maximized = FALSE;
+
     balsa_app.sw_width = 0;
     balsa_app.sw_height = 0;
+    balsa_app.sw_maximized = FALSE;
+
     balsa_app.toolbar_wrap_button_text = TRUE;
     balsa_app.pwindow_option = WHILERETR;
     balsa_app.wordwrap = TRUE;
@@ -403,8 +408,10 @@ balsa_app_init(void)
     balsa_app.notify_new_mail_sound = 1;
     balsa_app.notify_new_mail_dialog = 0;
 
+#if !GTK_CHECK_VERSION(2, 11, 0)
     /* Tooltips */
     balsa_app.tooltips = gtk_tooltips_new();
+#endif                          /* !GTK_CHECK_VERSION(2, 11, 0) */
 
     /* Local and IMAP */
     balsa_app.local_scan_depth = 1;
@@ -453,7 +460,7 @@ balsa_app_destroy(void)
 static gint
 check_new_messages_auto_cb(gpointer data)
 {
-    check_new_messages_real((GtkWidget *) NULL, data, TYPE_BACKGROUND);
+    check_new_messages_real(data, TYPE_BACKGROUND);
 
     if (balsa_app.debug)
         fprintf(stderr, "Auto-checked for new messages...\n");
@@ -768,7 +775,14 @@ gboolean
 balsa_find_iter_by_data(GtkTreeIter * iter , gpointer data)
 {
     struct balsa_find_iter_by_data_info bf;
-    GtkTreeModel *model = GTK_TREE_MODEL(balsa_app.mblist_tree_store);
+    GtkTreeModel *model;
+
+    /* We may call it from initial config, it's ok for
+       mblist_tree_store not to exist. */
+    if(!balsa_app.mblist_tree_store)
+        return FALSE;
+
+    model = GTK_TREE_MODEL(balsa_app.mblist_tree_store);
 
     bf.iter = iter;
     bf.data = data;
