@@ -19,6 +19,7 @@
  * 02111-1307, USA.
  */
 
+#include <string.h>
 #include <gnome.h>
 #include <gdk/gdkx.h>
 
@@ -48,6 +49,11 @@ typedef struct {
     const gchar * stock_id;
     GtkIconSize sizes[BALSA_PIXMAP_SIZES];
 } balsa_pixmap_t;
+
+typedef struct {
+    const gchar * def_id;
+    const gchar * fb_id;
+} pixmap_fallback_t;
     
 
 void
@@ -58,6 +64,10 @@ balsa_icon_create(const gchar ** data, GdkPixmap ** pmap, GdkBitmap ** bmap)
                                          bmap, 0, (gchar **) data);
 }
 
+#if GTK_CHECK_VERSION(2, 8, 0)
+static GHashTable *balsa_icon_table;
+#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
+
 static void
 load_balsa_pixmap(GtkIconTheme *icon_theme, GtkIconFactory *factory, 
 		  const balsa_pixmap_t *bpixmap)
@@ -66,9 +76,37 @@ load_balsa_pixmap(GtkIconTheme *icon_theme, GtkIconFactory *factory,
     GtkIconSet *icon_set;
     GError *error = NULL;
     gint n, width, height;
+    const gchar * use_id;
+    static pixmap_fallback_t fallback_id[] = {
+	{ "user-trash", GTK_STOCK_DELETE },
+	{ "user-trash-full", GTK_STOCK_DELETE },
+	{ "mail-attachment", "stock_attach" },
+	{ "emblem-important", "stock_mail-flag-for-followup"},
+	{ NULL, NULL } };
 
     BICONS_LOG("loading icon %s (stock id %s)", bpixmap->name,
 	       bpixmap->stock_id);
+
+    /* check if the icon theme knows the icon and try to fall back to an
+     * alternative name if not */
+    if (!gtk_icon_theme_has_icon(icon_theme, bpixmap->stock_id)) {
+	pixmap_fallback_t *fb = fallback_id;
+	while (fb->def_id && strcmp(fb->def_id, bpixmap->stock_id))
+	    fb++;
+	if (fb->def_id)
+	    use_id = fb->fb_id;
+	else {
+	    BICONS_ERR("icon %s unknown, no fallback", bpixmap->stock_id);
+	    return;
+	}
+    } else
+	use_id = bpixmap->stock_id;
+
+#if GTK_CHECK_VERSION(2, 8, 0)
+    g_hash_table_insert(balsa_icon_table, g_strdup(bpixmap->name), 
+                        g_strdup(use_id));
+#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
+
     if (!gtk_icon_size_lookup(bpixmap->sizes[0], &width, &height)) {
 	BICONS_ERR("failed: could not look up default icon size %d",
 		   bpixmap->sizes[0]);
@@ -76,7 +114,7 @@ load_balsa_pixmap(GtkIconTheme *icon_theme, GtkIconFactory *factory,
     }
 
     pixbuf = 
-	gtk_icon_theme_load_icon(icon_theme, bpixmap->stock_id, width,
+	gtk_icon_theme_load_icon(icon_theme, use_id, width,
 				 GTK_ICON_LOOKUP_USE_BUILTIN, &error);
     if (!pixbuf) {
 	BICONS_ERR("default size %d failed: %s", width, error->message);
@@ -92,7 +130,7 @@ load_balsa_pixmap(GtkIconTheme *icon_theme, GtkIconFactory *factory,
 	 n++) {
 	if (gtk_icon_size_lookup(bpixmap->sizes[n], &width, &height)) {
 	    pixbuf = 
-		gtk_icon_theme_load_icon(icon_theme, bpixmap->stock_id, width,
+		gtk_icon_theme_load_icon(icon_theme, use_id, width,
 					 GTK_ICON_LOOKUP_USE_BUILTIN, &error);
 	    if (!pixbuf) {
 		BICONS_ERR("additional size %d failed: %s", width,
@@ -149,7 +187,7 @@ register_balsa_pixmaps(void)
 	  { GTK_ICON_SIZE_LARGE_TOOLBAR, GTK_ICON_SIZE_MENU } },
 	{ BALSA_PIXMAP_MARK_ALL,	"balsa-mark-all",
 	  { GTK_ICON_SIZE_LARGE_TOOLBAR, GTK_ICON_SIZE_MENU } },
-	{ BALSA_PIXMAP_ATTACHMENT,	"stock_attach",
+	{ BALSA_PIXMAP_ATTACHMENT,	"mail-attachment",
 	  { GTK_ICON_SIZE_LARGE_TOOLBAR, GTK_ICON_SIZE_MENU } },
 	{ BALSA_PIXMAP_NEXT,		"balsa-next",
 	  { GTK_ICON_SIZE_LARGE_TOOLBAR, GTK_ICON_SIZE_MENU } },
@@ -216,6 +254,10 @@ register_balsa_pixmaps(void)
 	  { GTK_ICON_SIZE_MENU, GTK_ICON_SIZE_INVALID } },
         { BALSA_PIXMAP_MBOX_SENT,       "balsa-mbox-sent",
 	  { GTK_ICON_SIZE_MENU, GTK_ICON_SIZE_INVALID } },
+        { BALSA_PIXMAP_MBOX_TRASH,      "user-trash",
+	  { GTK_ICON_SIZE_MENU, GTK_ICON_SIZE_INVALID } },
+        { BALSA_PIXMAP_MBOX_TRASH_FULL, "user-trash-full",
+	  { GTK_ICON_SIZE_MENU, GTK_ICON_SIZE_INVALID } },
         { BALSA_PIXMAP_MBOX_TRAY_FULL,  "balsa-mbox-tray-full",
 	  { GTK_ICON_SIZE_MENU, GTK_ICON_SIZE_INVALID } },
         { BALSA_PIXMAP_MBOX_TRAY_EMPTY, "balsa-mbox-tray-empty",
@@ -230,7 +272,7 @@ register_balsa_pixmaps(void)
 	  { GTK_ICON_SIZE_MENU, GTK_ICON_SIZE_INVALID } },
         { BALSA_PIXMAP_INFO_NEW,        "stock_mail-unread",
 	  { GTK_ICON_SIZE_MENU, GTK_ICON_SIZE_INVALID } },
-	{ BALSA_PIXMAP_INFO_FLAGGED,    "stock_mail-flag-for-followup",
+	{ BALSA_PIXMAP_INFO_FLAGGED,    "emblem-important",
 	  { GTK_ICON_SIZE_MENU, GTK_ICON_SIZE_INVALID } },
 
 	};
@@ -239,6 +281,10 @@ register_balsa_pixmaps(void)
     GtkIconFactory *factory = gtk_icon_factory_new();
     GtkIconTheme *icon_theme = gtk_icon_theme_get_default();
 
+#if GTK_CHECK_VERSION(2, 8, 0)
+    balsa_icon_table =
+        g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
     gtk_icon_factory_add_default(factory);
     gtk_icon_theme_append_search_path(icon_theme, BALSA_DATA_PREFIX);
 
@@ -276,3 +322,13 @@ register_balsa_pixbufs(GtkWidget * widget)
 						 GTK_ICON_SIZE_MENU,
 						 NULL));
 }
+
+#if GTK_CHECK_VERSION(2, 8, 0)
+const gchar *
+balsa_icon_id(const gchar * name)
+{
+    const gchar *retval = g_hash_table_lookup(balsa_icon_table, name);
+
+    return retval ? retval : name;
+}
+#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
