@@ -31,7 +31,7 @@
 #include "balsa-index.h"
 #include "balsa-mblist.h"
 #include "libbalsa.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 #include "mailbox-node.h"
 #include "main-window.h"
 
@@ -65,6 +65,14 @@ enum {
     N_COLUMNS
 };
 
+/* signals */
+
+enum {
+    HAS_UNREAD_MAILBOX,
+    LAST_SIGNAL
+};
+static gint balsa_mblist_signals[LAST_SIGNAL] = { 0 };
+
 static GtkTargetEntry bmbl_drop_types[] = {
     {"x-application/x-message-list", GTK_TARGET_SAME_APP, TARGET_MESSAGES}
 };
@@ -172,6 +180,18 @@ bmbl_class_init(BalsaMBListClass * klass)
     object_class = (GtkObjectClass *) klass;
     widget_class = (GtkWidgetClass *) klass;
 
+    /* HAS_UNREAD_MAILBOX is emitted when the number of mailboxes with
+     * unread mail might have changed. */
+    balsa_mblist_signals[HAS_UNREAD_MAILBOX] =
+        g_signal_new("has-unread-mailbox",
+                     G_TYPE_FROM_CLASS(object_class),
+                     G_SIGNAL_RUN_FIRST,
+                     G_STRUCT_OFFSET(BalsaMBListClass, has_unread_mailbox),
+                     NULL, NULL,
+                     g_cclosure_marshal_VOID__INT,
+                     G_TYPE_NONE,
+                     1, G_TYPE_INT);
+
     /* GObject signals */
     o_class->set_property = bmbl_set_property;
     o_class->get_property = bmbl_get_property;
@@ -180,6 +200,9 @@ bmbl_class_init(BalsaMBListClass * klass)
     widget_class->drag_motion = bmbl_drag_motion;
     widget_class->popup_menu = bmbl_popup_menu;
 
+    /* Signals */
+    klass->has_unread_mailbox = NULL;
+
     /* Properties */
     g_object_class_install_property(o_class, PROP_SHOW_CONTENT_INFO,
                                     g_param_spec_boolean
@@ -187,23 +210,79 @@ bmbl_class_init(BalsaMBListClass * klass)
                                      FALSE, G_PARAM_READWRITE));
 }
 
+static void
+bmbl_set_property_node_style(GSList * list)
+{
+    GSList *l;
+    GtkTreePath *path;
+
+    gdk_threads_enter();
+
+    for (l = list; l; l = l->next) {
+        GtkTreeRowReference *reference = l->data;
+        path = gtk_tree_row_reference_get_path(reference);
+        if (path) {
+            GtkTreeModel *model;
+            GtkTreeIter iter;
+
+#if GTK_CHECK_VERSION(2, 8, 0)
+            model = gtk_tree_row_reference_get_model(reference);
+#else                           /* GTK_CHECK_VERSION(2, 8, 0) */
+            model = GTK_TREE_MODEL(balsa_app.mblist_tree_store);
+#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
+            if (gtk_tree_model_get_iter(model, &iter, path))
+                bmbl_node_style(model, &iter);
+            gtk_tree_path_free(path);
+        }
+        gtk_tree_row_reference_free(reference);
+    }
+
+    g_slist_free(list);
+    gdk_threads_leave();
+}
+
+static gboolean
+bmbl_set_property_foreach_func(GtkTreeModel * model, GtkTreePath * path,
+                               GtkTreeIter * iter, gpointer data)
+{
+    GSList **list = data;
+    *list =
+        g_slist_prepend(*list, gtk_tree_row_reference_new(model, path));
+    return FALSE;
+}
+
+#define BALSA_MBLIST_DISPLAY_INFO "balsa-mblist-display-info"
 static void
 bmbl_set_property(GObject * object, guint prop_id,
                           const GValue * value, GParamSpec * pspec)
 {
     BalsaMBList *mblist = BALSA_MBLIST(object);
     GtkTreeView *tree_view = GTK_TREE_VIEW(object);
+    GtkTreeModel *model = gtk_tree_view_get_model(tree_view);
     gboolean display_info;
     GtkTreeViewColumn *column;
+    GSList *list = NULL;
 
     switch (prop_id) {
     case PROP_SHOW_CONTENT_INFO:
         display_info = g_value_get_boolean(value);
         mblist->display_info = display_info;
+        gtk_tree_view_set_headers_visible(tree_view, display_info);
+        column = gtk_tree_view_get_column(tree_view, 0);
+        gtk_tree_view_column_set_sizing(column,
+                                        display_info ?
+                                        GTK_TREE_VIEW_COLUMN_FIXED :
+                                        GTK_TREE_VIEW_COLUMN_GROW_ONLY);
+        gtk_tree_view_column_set_resizable(column, display_info);
         column = gtk_tree_view_get_column(tree_view, 1);
         gtk_tree_view_column_set_visible(column, display_info);
         column = gtk_tree_view_get_column(tree_view, 2);
         gtk_tree_view_column_set_visible(column, display_info);
+        g_object_set_data(G_OBJECT(model), BALSA_MBLIST_DISPLAY_INFO,
+                          GINT_TO_POINTER(display_info));
+        gtk_tree_model_foreach(GTK_TREE_MODEL(balsa_app.mblist_tree_store),
+                               bmbl_set_property_foreach_func, &list);
+        bmbl_set_property_node_style(list);
         break;
 
     default:
@@ -280,7 +359,11 @@ bmbl_init(BalsaMBList * mblist)
     renderer = gtk_cell_renderer_pixbuf_new();
     gtk_tree_view_column_pack_start(column, renderer, FALSE);
     gtk_tree_view_column_set_attributes(column, renderer,
+#if GTK_CHECK_VERSION(2, 8, 0)
+                                        "icon-name", ICON_COLUMN,
+#else                           /* GTK_CHECK_VERSION(2, 8, 0) */
                                         "pixbuf", ICON_COLUMN,
+#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
                                         NULL);
     renderer = gtk_cell_renderer_text_new();
     gtk_tree_view_column_pack_start(column, renderer, FALSE);
@@ -404,7 +487,11 @@ balsa_mblist_get_store(void)
         balsa_app.mblist_tree_store =
             gtk_tree_store_new(N_COLUMNS,
                                G_TYPE_OBJECT,     /* MBNODE_COLUMN */
+#if GTK_CHECK_VERSION(2, 8, 0)
+                               G_TYPE_STRING,     /* ICON_COLUMN   */
+#else                           /* GTK_CHECK_VERSION(2, 8, 0) */
                                GDK_TYPE_PIXBUF,   /* ICON_COLUMN   */
+#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
                                G_TYPE_STRING,     /* NAME_COLUMN   */
                                PANGO_TYPE_WEIGHT, /* WEIGHT_COLUMN */
                                PANGO_TYPE_STYLE,  /* STYLE_COLUMN */
@@ -469,14 +556,30 @@ bmbl_tree_expand(GtkTreeView * tree_view, GtkTreeIter * iter,
     gtk_tree_model_get(model, iter, MBNODE_COLUMN, &mbnode, -1);
     balsa_mailbox_node_scan_children(mbnode);
 
-    if (!mbnode->mailbox)
+    if (!mbnode->mailbox) {
+#if GTK_CHECK_VERSION(2, 8, 0)
         gtk_tree_store_set(GTK_TREE_STORE(model), iter,
-                           ICON_COLUMN,   
-                           gtk_widget_render_icon
-                           (GTK_WIDGET(balsa_app.main_window),
-                            BALSA_PIXMAP_MBOX_DIR_OPEN,
-                            GTK_ICON_SIZE_MENU, NULL),
+                           ICON_COLUMN,
+                           balsa_icon_id(BALSA_PIXMAP_MBOX_DIR_OPEN),
                            -1);
+#else                           /* GTK_CHECK_VERSION(2, 8, 0) */
+        GdkPixbuf *pixbuf =
+            gtk_widget_render_icon(GTK_WIDGET(balsa_app.main_window),
+                                   BALSA_PIXMAP_MBOX_DIR_OPEN,
+                                   GTK_ICON_SIZE_MENU, NULL);
+        gtk_tree_store_set(GTK_TREE_STORE(model), iter,
+                           ICON_COLUMN, pixbuf,
+                           -1);
+        if(pixbuf) g_object_unref(pixbuf);
+        else { 
+            static int p=1;
+            if(p){
+                printf("icon %s not found.\n", BALSA_PIXMAP_MBOX_DIR_OPEN);
+                p = 0;
+            }
+        }
+#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
+    }
     g_object_unref(mbnode);
 
     if (gtk_tree_model_iter_children(model, &child_iter, iter)) {
@@ -544,14 +647,30 @@ bmbl_tree_collapse(GtkTreeView * tree_view, GtkTreeIter * iter,
 
     gtk_tree_model_get(model, iter, MBNODE_COLUMN, &mbnode, -1);
 
-    if (!mbnode->mailbox)
+    if (!mbnode->mailbox) {
+#if GTK_CHECK_VERSION(2, 8, 0)
         gtk_tree_store_set(GTK_TREE_STORE(model), iter,
-                           ICON_COLUMN,   
-                           gtk_widget_render_icon
-                           (GTK_WIDGET(balsa_app.main_window),
-                            BALSA_PIXMAP_MBOX_DIR_CLOSED,
-                            GTK_ICON_SIZE_MENU, NULL),
+                           ICON_COLUMN, 
+                           balsa_icon_id(BALSA_PIXMAP_MBOX_DIR_CLOSED),
                            -1);
+#else                           /* GTK_CHECK_VERSION(2, 8, 0) */
+        GdkPixbuf *pixbuf =
+            gtk_widget_render_icon(GTK_WIDGET(balsa_app.main_window),
+                                   BALSA_PIXMAP_MBOX_DIR_CLOSED,
+                                   GTK_ICON_SIZE_MENU, NULL);
+        gtk_tree_store_set(GTK_TREE_STORE(model), iter,
+                           ICON_COLUMN, pixbuf,
+                           -1);
+        if(pixbuf) g_object_unref(pixbuf);
+        else { 
+            static int p=1;
+            if(p){
+                printf("icon %s not found.\n", BALSA_PIXMAP_MBOX_DIR_CLOSED);
+                p = 0;
+            }
+        }
+#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
+    }
     g_object_unref(mbnode);
 
     bmbl_tree_collapse_helper(model, iter);
@@ -684,7 +803,12 @@ bmbl_do_popup(GtkTreeView * tree_view, GtkTreePath * path,
 
     menu = balsa_mailbox_node_get_context_menu(mbnode);
     g_object_ref(menu);
+#if GLIB_CHECK_VERSION(2, 10, 0)
+    g_object_ref_sink(menu);
+#else                           /* GLIB_CHECK_VERSION(2, 10, 0) */
+    g_object_ref(menu);
     gtk_object_sink(GTK_OBJECT(menu));
+#endif                          /* GLIB_CHECK_VERSION(2, 10, 0) */
     gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
 		   event_button, event_time);
     g_object_unref(menu);
@@ -747,16 +871,20 @@ bmbl_drag_cb(GtkWidget * widget, GdkDragContext * context,
     LibBalsaMailbox *mailbox;
     LibBalsaMailbox *orig_mailbox;
     BalsaIndex *orig_index;
+    GArray *selected;
 
     if (!selection_data || !selection_data->data)
 	/* Drag'n'drop is weird... */
 	return;
 
     orig_index = *(BalsaIndex **) selection_data->data;
-    if (orig_index->selected->len == 0)
+    selected = balsa_index_selected_msgnos_new(orig_index);
+    if (selected->len == 0) {
 	/* it is actually possible to drag from GtkTreeView when no rows
 	 * are selected: Disable preview for that. */
-       return; 
+        balsa_index_selected_msgnos_free(orig_index, selected);
+        return; 
+    }
 
     orig_mailbox = orig_index->mailbox_node->mailbox;
 
@@ -777,10 +905,11 @@ bmbl_drag_cb(GtkWidget * widget, GdkDragContext * context,
 
         /* cannot transfer to the originating mailbox */
         if (mailbox != NULL && mailbox != orig_mailbox)
-            balsa_index_transfer(orig_index, orig_index->selected, mailbox,
+            balsa_index_transfer(orig_index, selected, mailbox,
                                  context->action != GDK_ACTION_MOVE);
         gtk_tree_path_free(path);
     }
+    balsa_index_selected_msgnos_free(orig_index, selected);
 
     if (balsa_find_iter_by_data(&iter, orig_mailbox))
         gtk_tree_selection_select_iter(selection, &iter);
@@ -894,22 +1023,47 @@ struct update_mbox_data {
     gboolean notify;
 };
 static void bmbl_update_mailbox(GtkTreeStore * store,
-				LibBalsaMailbox * mailbox);
+                                LibBalsaMailbox * mailbox);
 static gboolean
 update_mailbox_idle(struct update_mbox_data *umd)
 {
     gdk_threads_enter();
+
     if (umd->mailbox) {
         g_object_remove_weak_pointer(G_OBJECT(umd->mailbox),
                                      (gpointer) & umd->mailbox);
-        if (balsa_app.mblist_tree_store) {
-            bmbl_update_mailbox(balsa_app.mblist_tree_store, umd->mailbox);
-            check_new_messages_count(umd->mailbox, umd->notify);
-        }
         g_object_set_data(G_OBJECT(umd->mailbox), "mblist-update", NULL);
+
+        if (balsa_app.mblist_tree_store) {
+            gboolean subscribed =
+                libbalsa_mailbox_get_subscribe(umd->mailbox) !=
+                LB_MAILBOX_SUBSCRIBE_NO;
+            bmbl_update_mailbox(balsa_app.mblist_tree_store, umd->mailbox);
+            check_new_messages_count(umd->mailbox, umd->notify
+                                     && subscribed);
+
+            if (subscribed) {
+                if (libbalsa_mailbox_get_unread(umd->mailbox) > 0)
+                    g_signal_emit(balsa_app.mblist,
+                                  balsa_mblist_signals[HAS_UNREAD_MAILBOX],
+                                  0, TRUE);
+                else {
+                    GList *unread_mailboxes =
+                        balsa_mblist_find_all_unread_mboxes(NULL);
+                    if (unread_mailboxes)
+                        g_list_free(unread_mailboxes);
+                    else
+                        g_signal_emit(balsa_app.mblist,
+                                      balsa_mblist_signals
+                                      [HAS_UNREAD_MAILBOX], 0, FALSE);
+                }
+            }
+        }
     }
-    gdk_threads_leave();
     g_free(umd);
+
+    gdk_threads_leave();
+
     return FALSE;
 }
 
@@ -917,14 +1071,28 @@ static void
 bmbl_mailbox_changed_cb(LibBalsaMailbox * mailbox, gpointer data)
 {
     struct update_mbox_data *umd;
-    g_return_if_fail(mailbox);
-    umd = g_new(struct update_mbox_data,1);
-    g_object_set_data(G_OBJECT(mailbox), "mblist-update", umd);
-    umd->mailbox = mailbox;
-    g_object_add_weak_pointer(G_OBJECT(mailbox), (gpointer) &umd->mailbox);
+
+    g_return_if_fail(LIBBALSA_IS_MAILBOX(mailbox));
+
+    /* Signal may be emitted without holding the gdk lock, so we'd
+     * better grab it. */
+    gdk_threads_enter();
+
+    umd = g_object_get_data(G_OBJECT(mailbox), "mblist-update");
+
+    if (!umd) {
+        umd = g_new(struct update_mbox_data, 1);
+        g_object_set_data(G_OBJECT(mailbox), "mblist-update", umd);
+        umd->mailbox = mailbox;
+        g_object_add_weak_pointer(G_OBJECT(mailbox),
+                                  (gpointer) & umd->mailbox);
+        g_idle_add((GSourceFunc) update_mailbox_idle, umd);
+    }
+
     umd->notify = (mailbox->state == LB_MAILBOX_STATE_OPEN
                    || mailbox->state == LB_MAILBOX_STATE_CLOSED);
-    g_idle_add((GSourceFunc)update_mailbox_idle, umd);
+
+    gdk_threads_leave();
 }
 
 /* public methods */
@@ -946,32 +1114,54 @@ balsa_mblist_get_selected_node(BalsaMBList * mbl)
 }
 
 /* mblist_find_all_unread_mboxes:
-   find all nodes and translate them to mailbox list 
-*/
+ * If mailbox == NULL, returns a list of all mailboxes with unread mail.
+ * If mailbox != NULL, returns a similar list, but including a NULL
+ * marker for the position of mailbox in the list.
+ * Trashbox is excluded.
+ */
+struct bmbl_find_all_unread_mboxes_info {
+    LibBalsaMailbox *mailbox;
+    GList *list;
+};
+
 static gboolean
 bmbl_find_all_unread_mboxes_func(GtkTreeModel * model, GtkTreePath * path,
                                  GtkTreeIter * iter, gpointer data)
 {
-    GList **r = data;
-    BalsaMailboxNode *mbnode;
+    struct bmbl_find_all_unread_mboxes_info *info = data;
+    BalsaMailboxNode *mbnode = NULL;
+    LibBalsaMailbox *mailbox;
 
     gtk_tree_model_get(model, iter, MBNODE_COLUMN, &mbnode, -1);
-    if (libbalsa_mailbox_get_unread(mbnode->mailbox) > 0)
-        *r = g_list_prepend(*r, mbnode->mailbox);
+    if(!mbnode) /* this node has no MBNODE associated at this time */
+        return FALSE;
+    mailbox = mbnode->mailbox;
     g_object_unref(mbnode);
 
+    if (mailbox
+        && (libbalsa_mailbox_get_subscribe(mailbox) !=
+            LB_MAILBOX_SUBSCRIBE_NO)) {
+        if (mailbox == info->mailbox)
+            info->list = g_list_prepend(info->list, NULL);
+        else if (libbalsa_mailbox_get_unread(mailbox) > 0)
+            info->list = g_list_prepend(info->list, mailbox);
+    }
+
     return FALSE;
 }
 
 GList *
-balsa_mblist_find_all_unread_mboxes(void)
+balsa_mblist_find_all_unread_mboxes(LibBalsaMailbox * mailbox)
 {
-    GList *res = NULL;
-    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(balsa_app.mblist));
+    struct bmbl_find_all_unread_mboxes_info info;
 
-    gtk_tree_model_foreach(model, bmbl_find_all_unread_mboxes_func, &res);
+    info.mailbox = mailbox;
+    info.list = NULL;
 
-    return res;
+    gtk_tree_model_foreach(GTK_TREE_MODEL(balsa_app.mblist_tree_store),
+                           bmbl_find_all_unread_mboxes_func, &info);
+
+    return g_list_reverse(info.list);
 }
 
 /* mblist_open_mailbox
@@ -1053,7 +1243,11 @@ get_lru_descendant(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter,
     gtk_tree_model_get(model, iter, MBNODE_COLUMN, &mbnode, -1);
     if(mbnode->mailbox && libbalsa_mailbox_is_open(mbnode->mailbox) &&
        (!dt->mbnode || (mbnode->last_use < dt->mbnode->last_use)) )
+    {
+        if (dt->mbnode)
+            g_object_unref(dt->mbnode);
         dt->mbnode = mbnode; 
+    }
 
     else g_object_unref(mbnode);
     return FALSE;
@@ -1079,7 +1273,9 @@ balsa_mblist_close_lru_peer_mbx(BalsaMBList * mblist,
     dt.mbnode = NULL;
     gtk_tree_model_foreach(model, get_lru_descendant, &dt);
     if(dt.mbnode) {
+        gdk_threads_enter();
         balsa_window_close_mbnode(balsa_app.main_window, dt.mbnode);
+        gdk_threads_leave();
         g_object_unref(dt.mbnode);
     }
     return dt.mbnode != NULL;
@@ -1137,8 +1333,12 @@ bmbl_real_disconnect_mbnode_signals(BalsaMailboxNode * mbnode,
 static gboolean
 bmbl_store_redraw_mbnode(GtkTreeIter * iter, BalsaMailboxNode * mbnode)
 {
-    const gchar *in;
-    gchar *name;
+    const gchar *icon;
+    const gchar *name;
+    gchar *tmp = NULL;
+#if !GTK_CHECK_VERSION(2, 8, 0)
+    GdkPixbuf *pixbuf;
+#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
     gboolean expose = FALSE;
 
     g_return_val_if_fail(mbnode, FALSE);
@@ -1149,25 +1349,25 @@ bmbl_store_redraw_mbnode(GtkTreeIter * iter, BalsaMailboxNode * mbnode)
 
 	if (LIBBALSA_IS_MAILBOX_POP3(mailbox)) {
 	    g_assert_not_reached();
-            in = NULL;
+            icon = NULL;
             name = NULL;
         } else {
 	    if(mailbox == balsa_app.draftbox)
-		in = BALSA_PIXMAP_MBOX_DRAFT;
+		icon = BALSA_PIXMAP_MBOX_DRAFT;
 	    else if(mailbox == balsa_app.inbox)
-		in = BALSA_PIXMAP_MBOX_IN;
+		icon = BALSA_PIXMAP_MBOX_IN;
 	    else if(mailbox == balsa_app.outbox)
-		in = BALSA_PIXMAP_MBOX_OUT;
+		icon = BALSA_PIXMAP_MBOX_OUT;
 	    else if(mailbox == balsa_app.sentbox)
-		in = BALSA_PIXMAP_MBOX_SENT;
+		icon = BALSA_PIXMAP_MBOX_SENT;
 	    else if(mailbox == balsa_app.trash)
-		in = GTK_STOCK_DELETE;
+		icon = BALSA_PIXMAP_MBOX_TRASH;
 	    else
-		in = (libbalsa_mailbox_total_messages(mailbox) > 0)
+		icon = (libbalsa_mailbox_total_messages(mailbox) > 0)
 		? BALSA_PIXMAP_MBOX_TRAY_FULL
                 : BALSA_PIXMAP_MBOX_TRAY_EMPTY;
 
-            name = g_strdup(mailbox->name);
+            name = mailbox->name;
 
             /* Make sure the show column is set before showing the
              * mailbox in the list. */
@@ -1175,9 +1375,17 @@ bmbl_store_redraw_mbnode(GtkTreeIter * iter, BalsaMailboxNode * mbnode)
 		libbalsa_mailbox_set_show(mailbox,
 					  (mailbox == balsa_app.sentbox
 					   || mailbox == balsa_app.draftbox
-					   || mailbox == balsa_app.outbox)
-					  ? LB_MAILBOX_SHOW_TO
-					  : LB_MAILBOX_SHOW_FROM);
+					   || mailbox == balsa_app.outbox) ?
+                                          LB_MAILBOX_SHOW_TO :
+                                          LB_MAILBOX_SHOW_FROM);
+            /* ...and whether to jump to new mail in this mailbox. */
+            if (libbalsa_mailbox_get_subscribe(mailbox) ==
+                LB_MAILBOX_SUBSCRIBE_UNSET)
+                libbalsa_mailbox_set_subscribe(mailbox,
+                                               mailbox ==
+                                               balsa_app.trash ?
+                                               LB_MAILBOX_SUBSCRIBE_NO :
+                                               LB_MAILBOX_SUBSCRIBE_YES);
 	}
 
 	if (!mailbox_changed_signal)
@@ -1189,29 +1397,52 @@ bmbl_store_redraw_mbnode(GtkTreeIter * iter, BalsaMailboxNode * mbnode)
 	    g_signal_connect(mbnode->mailbox, "changed",
 			     G_CALLBACK(bmbl_mailbox_changed_cb),
 			     NULL);
+            if (libbalsa_mailbox_get_unread(mailbox) > 0
+                && (libbalsa_mailbox_get_subscribe(mailbox) !=
+                    LB_MAILBOX_SUBSCRIBE_NO))
+                g_signal_emit(balsa_app.mblist,
+                              balsa_mblist_signals[HAS_UNREAD_MAILBOX],
+                              0, TRUE);
 	    /* If necessary, expand rows to expose this mailbox after
 	     * setting its mbnode in the tree-store. */
 	    expose = libbalsa_mailbox_get_exposed(mbnode->mailbox);
 	}
     } else {
 	/* new directory, but not a mailbox */
-	in = BALSA_PIXMAP_MBOX_DIR_CLOSED;
-        name = g_path_get_basename(mbnode->name);
+	icon = BALSA_PIXMAP_MBOX_DIR_CLOSED;
+        name = tmp = g_path_get_basename(mbnode->name);
     }
 
+#if GTK_CHECK_VERSION(2, 8, 0)
     gtk_tree_store_set(balsa_app.mblist_tree_store, iter,
                        MBNODE_COLUMN, mbnode,
-                       ICON_COLUMN,   
-                       gtk_widget_render_icon
-                           (GTK_WIDGET(balsa_app.main_window), in,
-                            GTK_ICON_SIZE_MENU, NULL),
+                       ICON_COLUMN, balsa_icon_id(icon),
                        NAME_COLUMN,   name,
                        WEIGHT_COLUMN, PANGO_WEIGHT_NORMAL,
                        STYLE_COLUMN, PANGO_STYLE_NORMAL,
                        UNREAD_COLUMN, "",
                        TOTAL_COLUMN,  "",
                        -1);
-    g_free(name);
+#else                           /* GTK_CHECK_VERSION(2, 8, 0) */
+    /* render icon may fail if the installation was incomplete */
+    pixbuf = gtk_widget_render_icon(GTK_WIDGET(balsa_app.main_window),
+                                    icon, GTK_ICON_SIZE_MENU, NULL);
+    gtk_tree_store_set(balsa_app.mblist_tree_store, iter,
+                       MBNODE_COLUMN, mbnode,
+                       ICON_COLUMN, pixbuf,
+                       NAME_COLUMN,   name,
+                       WEIGHT_COLUMN, PANGO_WEIGHT_NORMAL,
+                       STYLE_COLUMN, PANGO_STYLE_NORMAL,
+                       UNREAD_COLUMN, "",
+                       TOTAL_COLUMN,  "",
+                       -1);
+    if(pixbuf) g_object_unref(pixbuf);
+    else { 
+        static int p=1;
+        if(p){ printf("icon %s not found.\n", icon); p = 0; }
+    }
+#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
+    g_free(tmp);
 
     if (mbnode->mailbox) {
 	GtkTreeModel *model = GTK_TREE_MODEL(balsa_app.mblist_tree_store);
@@ -1287,6 +1518,9 @@ bmbl_node_style(GtkTreeModel * model, GtkTreeIter * iter)
     gchar *text_total = NULL;
 
     gtk_tree_model_get(model, iter, MBNODE_COLUMN, &mbnode, -1);
+    if (!mbnode || !mbnode->mailbox)
+        return;
+
     mailbox = mbnode->mailbox;
     unread_messages = libbalsa_mailbox_get_unread(mailbox);
     total_messages = libbalsa_mailbox_get_total(mailbox);
@@ -1295,38 +1529,61 @@ bmbl_node_style(GtkTreeModel * model, GtkTreeIter * iter)
     if (!(mailbox == balsa_app.sentbox || mailbox == balsa_app.outbox ||
           mailbox == balsa_app.draftbox || mailbox == balsa_app.trash)) {
         const gchar *icon;
+        const gchar *name;
+        gchar *tmp = NULL;
+        PangoWeight weight;
+#if !GTK_CHECK_VERSION(2, 8, 0)
+        GdkPixbuf *pixbuf;
+#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
 
+        /* Set the style appropriate for unread_messages; we do this
+         * even if the state hasn't changed, because we might be
+         * rerendering after hiding or showing the info columns. */
         if (unread_messages > 0) {
-            icon = BALSA_PIXMAP_MBOX_TRAY_FULL;
-            gtk_tree_store_set(GTK_TREE_STORE(model), iter,
-                               ICON_COLUMN,
-                               gtk_widget_render_icon
-                                   (GTK_WIDGET(balsa_app.main_window),
-                                    BALSA_PIXMAP_MBOX_TRAY_FULL,
-                                    GTK_ICON_SIZE_MENU, NULL),
-                               WEIGHT_COLUMN, PANGO_WEIGHT_BOLD, -1);
+            gboolean display_info;
 
+            icon = BALSA_PIXMAP_MBOX_TRAY_FULL;
+
+            display_info = GPOINTER_TO_INT(g_object_get_data
+                                           (G_OBJECT(model),
+                                            BALSA_MBLIST_DISPLAY_INFO));
+            name = (!display_info && total_messages >= 0) ?
+                (tmp = g_strdup_printf("%s (%d)", mailbox->name,
+                                      unread_messages))
+                : mailbox->name;
+
+            weight = PANGO_WEIGHT_BOLD;
             mbnode->style |= MBNODE_STYLE_NEW_MAIL;
         } else {
-            /* If the entry currently has the unread messages icon, set
-             * it back, otherwise we can ignore this. */
-            if (mbnode->style & MBNODE_STYLE_NEW_MAIL) {
-                if (mailbox == balsa_app.inbox)
-                    icon = BALSA_PIXMAP_MBOX_IN;
-                else
-                    icon = BALSA_PIXMAP_MBOX_TRAY_EMPTY;
-
-                gtk_tree_store_set(GTK_TREE_STORE(model), iter,
-                                   ICON_COLUMN,
-                                   gtk_widget_render_icon
-                                       (GTK_WIDGET(balsa_app.main_window),
-                                        icon, GTK_ICON_SIZE_MENU, NULL),
-                                   WEIGHT_COLUMN, PANGO_WEIGHT_NORMAL,
-                                   STYLE_COLUMN, PANGO_STYLE_NORMAL, -1);
-
-                mbnode->style &= ~MBNODE_STYLE_NEW_MAIL;
-            }
+            icon = (mailbox == balsa_app.inbox) ?
+                BALSA_PIXMAP_MBOX_IN : BALSA_PIXMAP_MBOX_TRAY_EMPTY;
+            name = mailbox->name;
+            weight = PANGO_WEIGHT_NORMAL;
+            mbnode->style &= ~MBNODE_STYLE_NEW_MAIL;
         }
+
+#if GTK_CHECK_VERSION(2, 8, 0)
+        gtk_tree_store_set(GTK_TREE_STORE(model), iter,
+                           ICON_COLUMN, balsa_icon_id(icon),
+                           NAME_COLUMN, name,
+                           WEIGHT_COLUMN, weight,
+                           -1);
+#else                           /* GTK_CHECK_VERSION(2, 8, 0) */
+        pixbuf = gtk_widget_render_icon(GTK_WIDGET(balsa_app.main_window),
+                                        icon, GTK_ICON_SIZE_MENU, NULL);
+        gtk_tree_store_set(GTK_TREE_STORE(model), iter,
+                           ICON_COLUMN, pixbuf,
+                           NAME_COLUMN, name,
+                           WEIGHT_COLUMN, weight,
+                           -1);
+        if(pixbuf) g_object_unref(pixbuf);
+        else { 
+            static int p=1;
+            if(p){ printf("icon %s not found.\n", icon); p = 0; }
+        }
+#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
+        g_free(tmp);
+
     }
     g_object_unref(mbnode);
 
@@ -1345,10 +1602,10 @@ bmbl_node_style(GtkTreeModel * model, GtkTreeIter * iter)
     g_free(text_total);
 
     /* Do the folder styles as well */
-    has_unread_child = mailbox->has_unread_messages;
+    has_unread_child = libbalsa_mailbox_get_unread(mailbox) > 0;
     while (gtk_tree_model_iter_parent(model, &parent, iter)) {
 	*iter = parent;
-	gtk_tree_model_get(model, &parent, 0, &mbnode, -1);
+	gtk_tree_model_get(model, &parent, MBNODE_COLUMN, &mbnode, -1);
 	if (!has_unread_child) {
 	    /* Check all the children of this parent. */
 	    GtkTreeIter child;
@@ -1356,7 +1613,7 @@ bmbl_node_style(GtkTreeModel * model, GtkTreeIter * iter)
 	    gtk_tree_model_iter_children(model, &child, &parent);
 	    do {
 		BalsaMailboxNode *mn;
-		gtk_tree_model_get(model, &child, 0, &mn, -1);
+		gtk_tree_model_get(model, &child, MBNODE_COLUMN, &mn, -1);
 		if (mn->style & (MBNODE_STYLE_NEW_MAIL |
 				 MBNODE_STYLE_UNREAD_CHILD))
 		    has_unread_child = TRUE;
@@ -2033,26 +2290,49 @@ balsa_mblist_mru_option_menu_get(GtkWidget * combo_box)
 void
 balsa_mblist_set_status_bar(LibBalsaMailbox * mailbox)
 {
-    guint total_messages = libbalsa_mailbox_total_messages(mailbox);
-    gchar *desc;
-    gchar *desc1, *desc2;
+    gint total_messages = libbalsa_mailbox_total_messages(mailbox);
+    gint unread_messages = mailbox->unread_messages;
+    gint hidden_messages;
+    GString *desc = g_string_new(NULL);
+    GtkStatusbar *statusbar;
+    guint context_id;
 
-    desc1 = g_strdup_printf(ngettext("Shown mailbox: %s with %d message, ",
-				     "Shown mailbox: %s with %d messages, ",
-				     total_messages),
-			    mailbox->name, total_messages);
-    /* xgettext: this is the second part of the message
-     * "Shown mailbox: %s with %d messages, %ld new". */
-    desc2 = g_strdup_printf(ngettext("%ld new", "%ld new",
-				     mailbox->unread_messages),
-			    mailbox->unread_messages);
-    desc = g_strconcat(desc1, desc2, NULL);
+    hidden_messages =
+        mailbox->msg_tree ? total_messages -
+        (g_node_n_nodes(mailbox->msg_tree, G_TRAVERSE_ALL) - 1) : 0;
 
-    gnome_appbar_set_default(balsa_app.appbar, desc);
+    /* xgettext: this is the first part of the message
+     * "Shown mailbox: %s with %d messages, %d new, %d hidden". */
+    g_string_append_printf(desc, _("Shown mailbox: %s "), mailbox->name);
+    if (total_messages > 0) {
+        /* xgettext: this is the second part of the message
+         * "Shown mailbox: %s with %d messages, %d new, %d hidden". */
+        g_string_append_printf(desc,
+                               ngettext("with %d message",
+                                        "with %d messages",
+                                        total_messages), total_messages);
+        if (unread_messages > 0)
+            /* xgettext: this is the third part of the message
+             * "Shown mailbox: %s with %d messages, %d new, %d hidden". */
+            g_string_append_printf(desc,
+                                   ngettext(", %d new", ", %d new",
+                                            unread_messages),
+                                   unread_messages);
+        if (hidden_messages > 0)
+            /* xgettext: this is the fourth part of the message
+             * "Shown mailbox: %s with %d messages, %d new, %d hidden". */
+            g_string_append_printf(desc,
+                                   ngettext(", %d hidden", ", %d hidden",
+                                            hidden_messages),
+                                   hidden_messages);
+    }
 
-    g_free(desc);
-    g_free(desc1);
-    g_free(desc2);
+    statusbar = GTK_STATUSBAR(balsa_app.main_window->statusbar);
+    context_id = gtk_statusbar_get_context_id(statusbar, "BalsaMBList message");
+    gtk_statusbar_pop(statusbar, context_id);
+    gtk_statusbar_push(statusbar, context_id, desc->str);
+
+    g_string_free(desc, TRUE);
 }
 
 static void
@@ -2075,14 +2355,12 @@ void
 balsa_mblist_mailbox_node_append(BalsaMailboxNode * root,
 				 BalsaMailboxNode * mbnode)
 {
-    gboolean is_sub_thread = libbalsa_am_i_subthread();
     GtkTreeModel *model;
     GtkTreeIter parent;
     GtkTreeIter *parent_iter = NULL;
     GtkTreeIter iter;
 
-    if (is_sub_thread)
-	gdk_threads_enter();
+    gdk_threads_enter();
 
     model = GTK_TREE_MODEL(balsa_app.mblist_tree_store);
     if (root && balsa_find_iter_by_data(&parent, root))
@@ -2090,11 +2368,24 @@ balsa_mblist_mailbox_node_append(BalsaMailboxNode * root,
     gtk_tree_store_append(balsa_app.mblist_tree_store, &iter, parent_iter);
     bmbl_store_redraw_mbnode(&iter, mbnode);
 
+    if (parent_iter) {
+        /* Check whether this node is exposed. */
+        GtkTreePath *parent_path =
+            gtk_tree_model_get_path(model, parent_iter);
+        if (gtk_tree_view_row_expanded(GTK_TREE_VIEW(balsa_app.mblist),
+                                       parent_path)
+            && !mbnode->scanned) {
+            /* Check this node for children. */
+            balsa_mailbox_node_append_subtree(mbnode);
+            mbnode->scanned = TRUE;
+        }
+        gtk_tree_path_free(parent_path);
+    }
+
     /* The tree-store owns mbnode. */
     g_object_unref(mbnode);
 
-    if (is_sub_thread)
-	gdk_threads_leave();
+    gdk_threads_leave();
 }
 
 /* Rerender a row after its properties have changed. */
