@@ -35,7 +35,7 @@
 #include "mime.h"
 #include "misc.h"
 #include "html.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 #include "balsa-mime-widget.h"
 #include "balsa-mime-widget-callbacks.h"
 #include "balsa-mime-widget-message.h"
@@ -265,8 +265,10 @@ bm_header_tl_buttons(BalsaMessage * bm)
     GtkWidget *vbox;
     GtkWidget *button;
 
+#if !GTK_CHECK_VERSION(2, 11, 0)
     /* the event box is needed to set the background correctly */
     GtkTooltips *tooltips = gtk_tooltips_new();
+#endif                          /* !GTK_CHECK_VERSION(2, 11, 0) */
 
     ebox = gtk_event_box_new();
 
@@ -283,8 +285,13 @@ bm_header_tl_buttons(BalsaMessage * bm)
     gtk_box_pack_start(GTK_BOX(hbox2), vbox, FALSE, FALSE, 0);
 
     button = gtk_button_new();
+#if GTK_CHECK_VERSION(2, 11, 0)
+    gtk_widget_set_tooltip_text(button,
+			        _("Check cryptographic signature"));
+#else                           /* GTK_CHECK_VERSION(2, 11, 0) */
     gtk_tooltips_set_tip(tooltips, button,
 			 _("Check cryptographic signature"), NULL);
+#endif                          /* GTK_CHECK_VERSION(2, 11, 0) */
     g_signal_connect(G_OBJECT(button), "focus_in_event",
 		     G_CALLBACK(balsa_mime_widget_limit_focus),
 		     (gpointer) bm);
@@ -304,8 +311,13 @@ bm_header_tl_buttons(BalsaMessage * bm)
     gtk_box_pack_start(GTK_BOX(hbox2), vbox, FALSE, FALSE, 0);
 
     button = gtk_button_new();
+#if GTK_CHECK_VERSION(2, 11, 0)
+    gtk_widget_set_tooltip_text(button,
+			        _("Select message part to display"));
+#else                           /* GTK_CHECK_VERSION(2, 11, 0) */
     gtk_tooltips_set_tip(tooltips, button,
 			 _("Select message part to display"), NULL);
+#endif                          /* GTK_CHECK_VERSION(2, 11, 0) */
     g_signal_connect(G_OBJECT(button), "focus_in_event",
 		     G_CALLBACK(balsa_mime_widget_limit_focus),
 		     (gpointer) bm);
@@ -314,7 +326,7 @@ bm_header_tl_buttons(BalsaMessage * bm)
 		     (gpointer) bm);
     gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
     gtk_container_add(GTK_CONTAINER(button), 
-		      gtk_image_new_from_stock("gnome-stock-attach", 
+		      gtk_image_new_from_stock(BALSA_PIXMAP_ATTACHMENT, 
 					       GTK_ICON_SIZE_LARGE_TOOLBAR));
     gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
     g_signal_connect(button, "clicked",
@@ -788,17 +800,23 @@ balsa_message_set(BalsaMessage * bm, LibBalsaMailbox * mailbox, guint msgno)
     }
 
     bm->message = message = libbalsa_mailbox_get_message(mailbox, msgno);
-
-    is_new = LIBBALSA_MESSAGE_IS_UNREAD(message);
-    if(!libbalsa_message_body_ref(message, TRUE, TRUE)) {
-	LibBalsaMailbox *mailbox = message->mailbox;
-	libbalsa_mailbox_check(mailbox);
+    if (!message) {
 	balsa_information(LIBBALSA_INFORMATION_WARNING,
                           _("Could not access message %ld "
                             "in mailbox \"%s\"."),
 			  msgno, mailbox->name);
+        return FALSE;
+    }
+
+    is_new = LIBBALSA_MESSAGE_IS_UNREAD(message);
+    if(!libbalsa_message_body_ref(message, TRUE, TRUE)) {
+	libbalsa_mailbox_check(mailbox);
         g_object_unref(bm->message);
         bm->message = NULL;
+	balsa_information(LIBBALSA_INFORMATION_WARNING,
+                          _("Could not access message %ld "
+                            "in mailbox \"%s\"."),
+			  msgno, mailbox->name);
         return FALSE;
     }
 
@@ -872,7 +890,7 @@ balsa_message_save_current_part(BalsaMessage * bm)
     g_return_if_fail(bm != NULL);
 
     if (bm->current_part)
-	balsa_mime_widget_ctx_menu_save(NULL, bm->current_part->body);
+	balsa_mime_widget_ctx_menu_save(GTK_WIDGET(bm), bm->current_part->body);
 }
 
 static gboolean
@@ -902,6 +920,9 @@ balsa_message_set_displayed_headers(BalsaMessage * bmessage,
     g_return_if_fail(bmessage != NULL);
     g_return_if_fail(sh >= HEADERS_NONE && sh <= HEADERS_ALL);
     
+    if (bmessage->shown_headers == sh)
+        return;
+
     bmessage->shown_headers = sh;
     
     if (bmessage->message) {
@@ -1326,7 +1347,8 @@ static void
 part_context_save_all_cb(GtkWidget * menu_item, GList * info_list)
 {
     while (info_list) {
-	balsa_mime_widget_ctx_menu_save(NULL, BALSA_PART_INFO(info_list->data)->body);
+	balsa_mime_widget_ctx_menu_save(menu_item,
+                                        BALSA_PART_INFO(info_list->data)->body);
         info_list = g_list_next(info_list);
     }
 }
@@ -1347,7 +1369,7 @@ part_context_dump_all_cb(GtkWidget * menu_item, GList * info_list)
 
     dump_dialog =
         gtk_file_chooser_dialog_new(_("Select folder for saving selected parts"),
-                                    GTK_WINDOW(balsa_app.main_window),
+                                    balsa_get_parent_window(menu_item),
                                     GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                     GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
@@ -1907,6 +1929,19 @@ balsa_message_current_part_widget(BalsaMessage * bmessage)
 	return NULL;
 }
 
+GtkWindow *
+balsa_get_parent_window(GtkWidget * widget)
+{
+    if (widget) {
+        GtkWidget *toplevel = gtk_widget_get_toplevel(widget);
+
+        if (GTK_WIDGET_TOPLEVEL(toplevel) && GTK_IS_WINDOW(toplevel))
+            return GTK_WINDOW(toplevel);
+    }
+
+    return GTK_WINDOW(balsa_app.main_window);
+}
+
 
 /*
  * This function informs the caller if the currently selected part 
