@@ -24,7 +24,7 @@
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
 #include "config.h"
 #include "balsa-app.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 #include "balsa-message.h"
 #include "balsa-mime-widget.h"
 #include "balsa-mime-widget-callbacks.h"
@@ -89,9 +89,7 @@ balsa_mime_widget_ctx_menu_vfs_cb(GtkWidget * menu_item,
 	if (app) {
 	    if (libbalsa_message_body_save_temporary(mime_body, &err)) {
 #if HAVE_GNOME_VFS29
-		gchar *uri =
-		    g_strconcat("file://", mime_body->temp_filename,
-				NULL);
+		gchar *uri = g_filename_to_uri(mime_body->temp_filename, NULL, NULL);
 		GList *uris = g_list_prepend(NULL, uri);
 		gnome_vfs_mime_application_launch(app, uris);
 		g_free(uri);
@@ -100,13 +98,14 @@ balsa_mime_widget_ctx_menu_vfs_cb(GtkWidget * menu_item,
 		gboolean tmp =
 		    (app->expects_uris ==
 		     GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS);
+		gchar *uri = g_filename_to_uri(mime_body->temp_filename, NULL, NULL);
 		gchar *exe_str =
-		    g_strdup_printf("%s \"%s%s\"", app->command,
-				    tmp ? "file://" : "",
-				    mime_body->temp_filename);
+		    g_strdup_printf("%s \"%s\"", app->command,
+				    tmp ? uri : mime_body->temp_filename);
 
 		gnome_execute_shell(NULL, exe_str);
 		fprintf(stderr, "Executed: %s\n", exe_str);
+		g_free(uri);
 		g_free(exe_str);
 #endif				/* HAVE_GNOME_VFS29 */
 	    } else {
@@ -124,8 +123,15 @@ balsa_mime_widget_ctx_menu_vfs_cb(GtkWidget * menu_item,
 }
 
 
+/** Pops up a "save part" dialog for a message part.
+
+    @param parent_widget the widget located in the window that is to
+    became a parent for the dialog box.
+
+    @param mime_body message part to be saved.
+*/
 void
-balsa_mime_widget_ctx_menu_save(GtkWidget * menu_item,
+balsa_mime_widget_ctx_menu_save(GtkWidget * parent_widget,
 				LibBalsaMessageBody * mime_body)
 {
     gchar *cont_type, *title;
@@ -138,9 +144,10 @@ balsa_mime_widget_ctx_menu_save(GtkWidget * menu_item,
 
     cont_type = libbalsa_message_body_get_mime_type(mime_body);
     title = g_strdup_printf(_("Save %s MIME Part"), cont_type);
+
     save_dialog =
 	gtk_file_chooser_dialog_new(title,
-				    GTK_WINDOW(balsa_app.main_window),
+                                    balsa_get_parent_window(parent_widget),
 				    GTK_FILE_CHOOSER_ACTION_SAVE,
 				    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 				    GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
