@@ -25,7 +25,7 @@
 #include "balsa-icons.h"
 #include "send.h"
 #include "rfc3156.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 #include "balsa-mime-widget.h"
 #include "balsa-mime-widget-callbacks.h"
 #include "balsa-mime-widget-message.h"
@@ -448,7 +448,7 @@ static void
 bm_header_ctx_menu_reply(GtkWidget * menu_item,
                          LibBalsaMessageBody *part)
 {
-    sendmsg_window_reply_embedded(NULL, part, SEND_REPLY);
+    sendmsg_window_reply_embedded(part, SEND_REPLY);
 }
 
 static void
