@@ -30,7 +30,7 @@
 #include <string.h>
 
 #include "libbalsa.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 #include "balsa-app.h"
 #include "information-dialog.h"
 
@@ -67,7 +67,10 @@ balsa_information_real(GtkWindow *parent, LibBalsaInformationType type,
 {
     BalsaInformationShow show;
     gchar * show_msg = g_strdup(msg);
-    
+
+    if (!balsa_app.main_window)
+        return;
+
     libbalsa_utf8_sanitize(&show_msg, balsa_app.convert_unknown_8bit, NULL);
     switch (type) {
     case LIBBALSA_INFORMATION_MESSAGE:
@@ -270,33 +273,42 @@ balsa_information_list(GtkWindow *parent, LibBalsaInformationType type,
                                  gtk_text_buffer_get_insert(buffer),
                                  0, FALSE, 0, 0);
 
-    if (balsa_app.appbar) {
-        gchar *line = g_strdup(msg);
-        g_strdelimit(line, "\n", ' ');
-	gnome_appbar_set_status(balsa_app.appbar, line);
+    if (balsa_app.main_window) {
+        gchar *line;
+        GtkStatusbar *statusbar;
+        guint context_id;
+
+        statusbar = GTK_STATUSBAR(balsa_app.main_window->statusbar);
+        context_id = gtk_statusbar_get_context_id(statusbar, "Information list");
+        gtk_statusbar_pop(statusbar, context_id);
+
+        line = g_strdup(msg);
+        g_strdelimit(line, "\r\n", ' ');
+        gtk_statusbar_push(statusbar, context_id, line);
         g_free(line);
     }
 }
 
 static guint bar_timeout_id = 0;
-static unsigned push_cnt = 0;
 static gboolean
 status_bar_refresh(gpointer data)
 {
-    gboolean retval = FALSE;
-
     gdk_threads_enter();
-    if (balsa_app.appbar) {
-        gnome_appbar_pop(balsa_app.appbar);
-        push_cnt--;
-        if(push_cnt == 0)
-            bar_timeout_id = 0;
-        else
-            retval = TRUE;
+
+    if (balsa_app.main_window) {
+        GtkStatusbar *statusbar;
+        guint context_id;
+
+        statusbar = GTK_STATUSBAR(balsa_app.main_window->statusbar);
+        context_id = gtk_statusbar_get_context_id(statusbar, "Information bar");
+        gtk_statusbar_pop(statusbar, context_id);
     }
+
+    bar_timeout_id = 0;
+
     gdk_threads_leave();
 
-    return retval;
+    return FALSE;
 }
 
 static void
@@ -304,21 +316,26 @@ balsa_information_bar(GtkWindow *parent, LibBalsaInformationType type,
                       const char *msg)
 {
     gchar *line;
+    GtkStatusbar *statusbar;
+    guint context_id;
 
-    if (!balsa_app.appbar)
+    if (!balsa_app.main_window)
         return;
 
+    statusbar = GTK_STATUSBAR(balsa_app.main_window->statusbar);
+    context_id = gtk_statusbar_get_context_id(statusbar, "Information bar");
+
+    /* First clear any current message. */
+    if (bar_timeout_id) {
+        gtk_statusbar_pop(statusbar, context_id);
+        g_source_remove(bar_timeout_id);
+    }
+
     line = g_strdup(msg);
-    g_strdelimit(line, "\n", ' ');
-    /* we used to have _set_status() here but it got over-written by
-       set_default(). _push() seems to be more persistent. */
-    gnome_appbar_push(balsa_app.appbar, line); push_cnt++;
+    g_strdelimit(line, "\r\n", ' ');
+    gtk_statusbar_push(statusbar, context_id, line);
     g_free(line);
 
-    if (bar_timeout_id)
-        /* Reschedule the refresh, so this message gets its full
-         * 4 seconds. */
-        g_source_remove(bar_timeout_id);
     bar_timeout_id = g_timeout_add(4000, status_bar_refresh, NULL);
 }
 
