@@ -53,13 +53,13 @@
 
 #include "libbalsa.h"
 #include "imap-server.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 
 struct _BalsaMailboxConfView {
     GtkWindow *window;
     GtkWidget *identity_combo_box;
-    GtkWidget *show_from;
     GtkWidget *show_to;
+    GtkWidget *subscribe;
 #ifdef HAVE_GPGME
     GtkWidget *chk_crypt;
 #endif
@@ -124,6 +124,11 @@ static void fill_in_imap_data(MailboxConfWindow *mcw, gchar ** name,
 static void update_imap_mailbox(MailboxConfWindow *mcw);
 
 static void update_pop_mailbox(MailboxConfWindow *mcw);
+static BalsaMailboxConfView
+    *mailbox_conf_view_new_with_size_group(LibBalsaMailbox * mailbox,
+                                           GtkWindow * window,
+                                           GtkWidget * table, gint row,
+                                           GtkSizeGroup * size_group);
 
 /* pages */
 static GtkWidget *create_dialog(MailboxConfWindow *mcw);
@@ -201,19 +206,23 @@ balsa_server_conf_get_advanced_widget(BalsaServerConf *bsc, LibBalsaServer *s,
         { N_("Required"),    LIBBALSA_TLS_REQUIRED }
     };
     GtkWidget *label;
+    GtkWidget *hbox;
     gboolean use_ssl = s && s->use_ssl;
 
-    bsc->table = GTK_TABLE(gtk_table_new(3+extra_rows, 2, FALSE));
+    hbox = gtk_hbox_new(FALSE, 0);
+
+    bsc->table = GTK_TABLE(libbalsa_create_table(3 + extra_rows, 2));
+    gtk_container_set_border_width(GTK_CONTAINER(bsc->table), 12);
+    gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(bsc->table),
+                       FALSE, FALSE, 0);
+
     bsc->used_rows = 0;
 
     bsc->use_ssl = balsa_server_conf_add_checkbox(bsc, _("Use _SSL"));
     if(use_ssl)
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(bsc->use_ssl), TRUE);
 
-    label = gtk_label_new_with_mnemonic(_("Use _TLS"));
-    gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
-    gtk_table_attach(bsc->table, label, 0, 1, 1, 2,
-		     GTK_FILL, GTK_FILL, 0, 0);
+    label = libbalsa_create_label(_("Use _TLS:"), GTK_WIDGET(bsc->table), 1);
 
     bsc->tls_option = gtk_combo_box_new_text();
     mailbox_conf_combo_box_make(GTK_COMBO_BOX(bsc->tls_option),
@@ -221,7 +230,7 @@ balsa_server_conf_get_advanced_widget(BalsaServerConf *bsc, LibBalsaServer *s,
     gtk_combo_box_set_active(GTK_COMBO_BOX(bsc->tls_option),
                              s ? s->tls_mode : LIBBALSA_TLS_ENABLED);
     gtk_table_attach(bsc->table, bsc->tls_option, 1, 2, 1, 2,
-		     GTK_FILL, GTK_FILL, 5, 5);
+		     GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), bsc->tls_option);
 
     g_signal_connect(G_OBJECT (bsc->use_ssl), "toggled",
@@ -229,26 +238,16 @@ balsa_server_conf_get_advanced_widget(BalsaServerConf *bsc, LibBalsaServer *s,
     gtk_widget_show_all(GTK_WIDGET(bsc->table));
     bsc->used_rows = 2;
     gtk_widget_set_sensitive(bsc->tls_option, !use_ssl);
-    return (GtkWidget*)bsc->table;
+
+    return hbox;
 }
 
 GtkWidget*
 balsa_server_conf_add_checkbox(BalsaServerConf *bsc,
                                const char *label)
 {
-    GtkWidget *checkbox;
-    GtkWidget *lbl = gtk_label_new_with_mnemonic(label);
-    gtk_misc_set_alignment(GTK_MISC(lbl), 1.0, 0.5);
-    gtk_table_attach(bsc->table, lbl, 0, 1,
-                     bsc->used_rows, bsc->used_rows+1,
-		     GTK_FILL, GTK_FILL, 5, 5);
-    checkbox = gtk_check_button_new();
-    gtk_table_attach(bsc->table, checkbox, 1, 2,
-                     bsc->used_rows, bsc->used_rows+1,
-		     GTK_FILL, GTK_FILL, 5, 5);
-    gtk_label_set_mnemonic_widget(GTK_LABEL(lbl), checkbox);
-    bsc->used_rows++;
-    return checkbox;
+    return libbalsa_create_check(label, GTK_WIDGET(bsc->table),
+                                 bsc->used_rows++, FALSE);
 }
 
 GtkWidget*
@@ -257,18 +256,15 @@ balsa_server_conf_add_spinner(BalsaServerConf *bsc,
                               gint initial_value)
 {
     GtkWidget *spin_button;
-    GtkWidget *lbl = gtk_label_new_with_mnemonic(label);
-    gtk_misc_set_alignment(GTK_MISC(lbl), 1.0, 0.5);
-    gtk_table_attach(bsc->table, lbl, 0, 1,
-                     bsc->used_rows, bsc->used_rows+1,
-		     GTK_FILL, GTK_FILL, 5, 5);
+    GtkWidget *lbl = libbalsa_create_label(label, GTK_WIDGET(bsc->table),
+                                           bsc->used_rows);
     spin_button = gtk_spin_button_new_with_range(lo, hi, step);
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin_button),
 			      (float) initial_value);
 
     gtk_table_attach(bsc->table, spin_button, 1, 2,
                      bsc->used_rows, bsc->used_rows+1,
-		     GTK_FILL, GTK_FILL, 5, 5);
+		     GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
     gtk_label_set_mnemonic_widget(GTK_LABEL(lbl), spin_button);
     bsc->used_rows++;
     return spin_button;
@@ -362,8 +358,10 @@ mailbox_conf_edit_cb(GtkWidget * widget, gpointer data)
 {
     BalsaMailboxNode *mbnode = 
         balsa_mblist_get_selected_node(balsa_app.mblist);
-    balsa_mailbox_node_show_prop_dialog(mbnode);
-    g_object_unref(mbnode);
+    if (mbnode) {
+        balsa_mailbox_node_show_prop_dialog(mbnode);
+        g_object_unref(mbnode);
+    }
 }
 
 /* END OF COMMONLY USED CALLBACKS SECTION ------------------------ */
@@ -497,6 +495,7 @@ mailbox_conf_delete(BalsaMailboxNode * mbnode)
 }
 
 #define MCW_RESPONSE 1
+#define BALSA_MAILBOX_CONF_DIALOG "balsa-mailbox-conf-dialog"
 static void
 conf_response_cb(GtkDialog* dialog, gint response, MailboxConfWindow * mcw)
 {
@@ -504,6 +503,9 @@ conf_response_cb(GtkDialog* dialog, gint response, MailboxConfWindow * mcw)
     case MCW_RESPONSE: mcw->ok_handler(mcw); 
         /* fall through */
     default:
+        if (mcw->mailbox)
+            g_object_set_data(G_OBJECT(mcw->mailbox),
+                              BALSA_MAILBOX_CONF_DIALOG, NULL);
         gtk_widget_destroy(GTK_WIDGET(dialog));
         /* fall through */
     case 0:
@@ -511,13 +513,14 @@ conf_response_cb(GtkDialog* dialog, gint response, MailboxConfWindow * mcw)
     }
 }
 
-static void
+static GtkWidget *
 run_mailbox_conf(BalsaMailboxNode* mbnode, GtkType mailbox_type, 
 		 gboolean update)
 {
     MailboxConfWindow* mcw;
 
-    g_return_if_fail(g_type_is_a(mailbox_type, LIBBALSA_TYPE_MAILBOX));
+    g_return_val_if_fail(g_type_is_a(mailbox_type, LIBBALSA_TYPE_MAILBOX),
+                         NULL);
 
     mcw = g_new0(MailboxConfWindow, 1);
 
@@ -546,6 +549,8 @@ run_mailbox_conf(BalsaMailboxNode* mbnode, GtkType mailbox_type,
     g_signal_connect(G_OBJECT(mcw->window), "response", 
                      G_CALLBACK(conf_response_cb), mcw);
     gtk_widget_show_all(GTK_WIDGET(mcw->window));
+
+    return GTK_WIDGET(mcw->window);
 }
 /*
  * Brings up dialog to configure a new mailbox of type mailbox_type.
@@ -554,18 +559,39 @@ run_mailbox_conf(BalsaMailboxNode* mbnode, GtkType mailbox_type,
 void
 mailbox_conf_new(GtkType mailbox_type)
 {
-    run_mailbox_conf(NULL, mailbox_type, FALSE);
+    static GtkWidget *dialog;
+
+    if (dialog) {
+        gdk_window_raise(dialog->window);
+        return;
+    }
+
+    dialog = run_mailbox_conf(NULL, mailbox_type, FALSE);
+    g_object_add_weak_pointer(G_OBJECT(dialog), (gpointer) &dialog);
 }
 
 /*
  * Edit an existing mailboxes properties
  */
 void
-mailbox_conf_edit(BalsaMailboxNode *mbnode)
+mailbox_conf_edit(BalsaMailboxNode * mbnode)
 {
+    GtkWidget *dialog;
+
     g_return_if_fail(LIBBALSA_IS_MAILBOX(mbnode->mailbox));
-    run_mailbox_conf(mbnode, G_OBJECT_TYPE(G_OBJECT(mbnode->mailbox)),
-                     TRUE);
+
+    dialog = g_object_get_data(G_OBJECT(mbnode->mailbox),
+                               BALSA_MAILBOX_CONF_DIALOG);
+    if (dialog) {
+        gdk_window_raise(dialog->window);
+        return;
+    }
+
+    dialog =
+        run_mailbox_conf(mbnode, G_OBJECT_TYPE(G_OBJECT(mbnode->mailbox)),
+                         TRUE);
+    g_object_set_data(G_OBJECT(mbnode->mailbox), BALSA_MAILBOX_CONF_DIALOG,
+                      dialog);
 }
 
 /*
@@ -581,26 +607,14 @@ mailbox_conf_set_values(MailboxConfWindow *mcw)
     g_return_if_fail(LIBBALSA_IS_MAILBOX(mailbox));
 
     if (mcw->mailbox_name && mailbox->name)
-	gtk_entry_set_text(GTK_ENTRY(mcw->mailbox_name),
-			   mailbox->name);
+	gtk_entry_set_text(GTK_ENTRY(mcw->mailbox_name), mailbox->name);
 
     if (LIBBALSA_IS_MAILBOX_LOCAL(mailbox)) {
-	GtkFileChooser *chooser = GTK_FILE_CHOOSER(mcw->window);
-	LibBalsaMailboxLocal *local = LIBBALSA_MAILBOX_LOCAL(mailbox);
-	const gchar *path = libbalsa_mailbox_local_get_path(local);
-
-        if (mailbox->url && gtk_file_chooser_set_filename(chooser, path)
-            && gtk_file_chooser_get_action(chooser)
-            == GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER) {
-	    /* Name entry will be blank unless we set it. */
-            gchar *folder;
-            gchar *utf8name;
-	    
-            folder = g_path_get_basename(path);
-            utf8name = g_filename_to_utf8(folder, -1, NULL, NULL, NULL);
-            g_free(folder);
-            gtk_file_chooser_set_current_name(chooser, utf8name);
-            g_free(utf8name);
+        if (mailbox->url) {
+            GtkFileChooser *chooser = GTK_FILE_CHOOSER(mcw->window);
+	    LibBalsaMailboxLocal *local = LIBBALSA_MAILBOX_LOCAL(mailbox);
+            const gchar *path = libbalsa_mailbox_local_get_path(local);
+            gtk_file_chooser_set_filename(chooser, path);
         }
     } else if (LIBBALSA_IS_MAILBOX_POP3(mailbox)) {
 	LibBalsaMailboxPop3 *pop3;
@@ -1047,18 +1061,19 @@ create_local_mailbox_dialog(MailboxConfWindow *mcw)
 {
     GtkWidget *dialog;
     GtkWidget *table;
-    GtkWidget *label;
+    GtkWidget *label = NULL;
     gint row = -1;
     GtkFileChooserAction action;
     GtkWidget *entry = NULL;
+    GtkSizeGroup *size_group;
 
-    table = gtk_table_new(3, 2, FALSE);
+    table = libbalsa_create_table(3, 2);
 
     /* mailbox name */
     if(mcw->mailbox && mcw->mailbox->config_prefix) {
-        label = create_label(_("Mailbox _Name:"), table, ++row);
+        label = libbalsa_create_label(_("_Mailbox Name:"), table, ++row);
         mcw->mailbox_name = 
-            create_entry(mcw->window, table,
+            libbalsa_create_entry(table,
                          GTK_SIGNAL_FUNC(check_for_blank_fields),
                          mcw, row, NULL, label);
     } else mcw->mailbox_name = NULL;
@@ -1073,6 +1088,11 @@ create_local_mailbox_dialog(MailboxConfWindow *mcw)
                                     mcw->ok_button_name, MCW_RESPONSE,
                                     GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                                     NULL);
+
+    size_group = libbalsa_create_size_group(dialog);
+    if (label)
+        gtk_size_group_add_widget(size_group, label);
+
     gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(dialog), table);
     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog),
 	                                balsa_app.local_mail_directory);
@@ -1084,8 +1104,9 @@ create_local_mailbox_dialog(MailboxConfWindow *mcw)
                          G_CALLBACK(check_for_blank_fields), mcw);
 
     mcw->view_info =
-        mailbox_conf_view_new(mcw->mailbox, GTK_WINDOW(dialog), table,
-                              ++row);
+        mailbox_conf_view_new_with_size_group(mcw->mailbox,
+                                              GTK_WINDOW(dialog),
+                                              table, ++row, size_group);
 
     return dialog;
 }
@@ -1110,57 +1131,58 @@ create_pop_mailbox_dialog(MailboxConfWindow *mcw)
     GtkWidget *notebook, *table, *label, *advanced;
 
     notebook = gtk_notebook_new();
-    table = gtk_table_new(9, 2, FALSE);
+    table = libbalsa_create_table(9, 2);
+    gtk_container_set_border_width(GTK_CONTAINER(table), 12);
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), table,
                              gtk_label_new_with_mnemonic(_("_Basic")));
 
     /* mailbox name */
-    label = create_label(_("Mailbox _Name:"), table, 0);
-    mcw->mailbox_name = create_entry(mcw->window, table,
+    label = libbalsa_create_label(_("Mailbox _Name:"), table, 0);
+    mcw->mailbox_name = libbalsa_create_entry(table,
 				     GTK_SIGNAL_FUNC(check_for_blank_fields),
 				     mcw, 0, NULL, label);
     /* pop server */
-    label = create_label(_("_Server:"), table, 1);
-    mcw->mb_data.pop3.bsc.server = create_entry(mcw->window, table,
+    label = libbalsa_create_label(_("_Server:"), table, 1);
+    mcw->mb_data.pop3.bsc.server = libbalsa_create_entry(table,
 				     GTK_SIGNAL_FUNC(check_for_blank_fields),
 				     mcw, 1, "localhost", label);
     mcw->mb_data.pop3.bsc.default_ports = POP3_DEFAULT_PORTS;
 
 
     /* username  */
-    label= create_label(_("Use_r name:"), table, 3);
-    mcw->mb_data.pop3.username = create_entry(mcw->window, table,
+    label= libbalsa_create_label(_("Use_r name:"), table, 2);
+    mcw->mb_data.pop3.username = libbalsa_create_entry(table,
 				     GTK_SIGNAL_FUNC(check_for_blank_fields),
-				     mcw, 3, g_get_user_name(), label);
+				     mcw, 2, g_get_user_name(), label);
 
     /* password field */
-    label = create_label(_("Pass_word:"), table, 4);
-    mcw->mb_data.pop3.password = create_entry(mcw->window, table, 
-					      NULL, NULL, 4, NULL, label);
+    label = libbalsa_create_label(_("Pass_word:"), table, 3);
+    mcw->mb_data.pop3.password = libbalsa_create_entry(table, 
+					      NULL, NULL, 3, NULL, label);
     gtk_entry_set_visibility(GTK_ENTRY(mcw->mb_data.pop3.password), FALSE);
 
     /* toggle for deletion from server */
     mcw->mb_data.pop3.delete_from_server = 
-	create_check(mcw->window, 
+	libbalsa_create_check(
 		     _("_Delete messages from server after download"),
-		     table, 6, TRUE);
-
-    /* Procmail */
-    mcw->mb_data.pop3.filter = 
-	create_check(mcw->window, _("_Filter messages through procmail"),
-		     table, 7, FALSE);
-    g_signal_connect(G_OBJECT(mcw->mb_data.pop3.filter), "toggled", 
-                     G_CALLBACK(pop3_enable_filter_cb), mcw);
-    label = create_label(_("Fi_lter Command:"), table, 8);
-    mcw->mb_data.pop3.filter_cmd =
-	create_entry(mcw->window, table,
-		     GTK_SIGNAL_FUNC(check_for_blank_fields),
-		     mcw, 8, "procmail -f -", label);
+		     table, 4, TRUE);
 
     /* toggle for check */
     mcw->mb_data.pop3.check = 
-	create_check(mcw->window, _("_Enable check for new mail"), 
-		     table, 9, TRUE);
+	libbalsa_create_check(_("_Enable check for new mail"), 
+		     table, 5, TRUE);
+
+    /* Procmail */
+    mcw->mb_data.pop3.filter = 
+	libbalsa_create_check(_("_Filter messages through procmail"),
+		     table, 6, FALSE);
+    g_signal_connect(G_OBJECT(mcw->mb_data.pop3.filter), "toggled", 
+                     G_CALLBACK(pop3_enable_filter_cb), mcw);
+    label = libbalsa_create_label(_("Fi_lter Command:"), table, 7);
+    mcw->mb_data.pop3.filter_cmd =
+	libbalsa_create_entry(table,
+		     GTK_SIGNAL_FUNC(check_for_blank_fields),
+		     mcw, 7, "procmail -f -", label);
 
     advanced =
         balsa_server_conf_get_advanced_widget(&mcw->mb_data.pop3.bsc,
@@ -1211,51 +1233,52 @@ create_imap_mailbox_dialog(MailboxConfWindow *mcw)
     gint row = -1;
 
     notebook = gtk_notebook_new();
-    table = gtk_table_new(8, 2, FALSE);
+    table = libbalsa_create_table(8, 2);
+    gtk_container_set_border_width(GTK_CONTAINER(table), 12);
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), table,
                              gtk_label_new_with_mnemonic(_("_Basic")));
 
     /* mailbox name */
-    label = create_label(_("Mailbox _Name:"), table, ++row);
-    mcw->mailbox_name =  create_entry(mcw->window, table,
+    label = libbalsa_create_label(_("Mailbox _Name:"), table, ++row);
+    mcw->mailbox_name =  libbalsa_create_entry(table,
                                       GTK_SIGNAL_FUNC(check_for_blank_fields),
                                       mcw, row, NULL, label);
 
     /* imap server */
-    label = create_label(_("_Server:"), table, ++row);
+    label = libbalsa_create_label(_("_Server:"), table, ++row);
     mcw->mb_data.imap.bsc.server = 
-	create_entry(mcw->window, table,
+	libbalsa_create_entry(table,
 		     GTK_SIGNAL_FUNC(check_for_blank_fields),
 		     mcw, row, "localhost", label);
     mcw->mb_data.imap.bsc.default_ports = IMAP_DEFAULT_PORTS;
 
     /* username  */
-    label = create_label(_("_Username:"), table, ++row);
+    label = libbalsa_create_label(_("_Username:"), table, ++row);
     mcw->mb_data.imap.username = 
-	create_entry(mcw->window, table,
+	libbalsa_create_entry(table,
 		     GTK_SIGNAL_FUNC(check_for_blank_fields),
 		     mcw, row, g_get_user_name(), label);
 
     /* toggle for anonymous password */
     mcw->mb_data.imap.anonymous = 
-	create_check(mcw->window, _("_Anonymous access"), table, ++row,
+	libbalsa_create_check(_("_Anonymous access"), table, ++row,
 		     FALSE);
     g_signal_connect(G_OBJECT(mcw->mb_data.imap.anonymous), "toggled",
                      G_CALLBACK(anon_toggle_cb), mcw);
     /* toggle for remember password */
     mcw->mb_data.imap.remember = 
-	create_check(mcw->window, _("_Remember Password"), table, ++row,
+	libbalsa_create_check(_("_Remember Password"), table, ++row,
 		     FALSE);
     g_signal_connect(G_OBJECT(mcw->mb_data.imap.remember), "toggled",
                      G_CALLBACK(remember_toggle_cb), mcw);
 
    /* password field */
-    label = create_label(_("Pass_word:"), table, ++row);
+    label = libbalsa_create_label(_("Pass_word:"), table, ++row);
     mcw->mb_data.imap.password = 
-	create_entry(mcw->window, table, NULL, NULL, row, NULL, label);
+	libbalsa_create_entry(table, NULL, NULL, row, NULL, label);
     gtk_entry_set_visibility(GTK_ENTRY(mcw->mb_data.imap.password), FALSE);
 
-    label = create_label(_("F_older Path:"), table, ++row);
+    label = libbalsa_create_label(_("F_older Path:"), table, ++row);
 
     mcw->mb_data.imap.folderpath = entry = gtk_entry_new();
     gtk_entry_set_text(GTK_ENTRY(mcw->mb_data.imap.folderpath), "INBOX");
@@ -1268,7 +1291,7 @@ create_imap_mailbox_dialog(MailboxConfWindow *mcw)
                      G_CALLBACK(check_for_blank_fields), mcw);
 
     gtk_table_attach(GTK_TABLE(table), entry, 1, 2, row, row + 1,
-		     GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 10);
+		     GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
 
     advanced =
         balsa_server_conf_get_advanced_widget(&mcw->mb_data.imap.bsc,
@@ -1323,13 +1346,15 @@ mailbox_conf_view_free(BalsaMailboxConfView *view_info)
  * table:       the table in which to place the widgets;
  * row:         the row of the table in which to start.
  */
-BalsaMailboxConfView *
-mailbox_conf_view_new(LibBalsaMailbox * mailbox,
-                      GtkWindow * window, GtkWidget * table, gint row)
+static BalsaMailboxConfView *
+mailbox_conf_view_new_with_size_group(LibBalsaMailbox * mailbox,
+                                      GtkWindow * window,
+                                      GtkWidget * table, gint row,
+                                      GtkSizeGroup * size_group)
 {
+    GtkWidget *label;
     BalsaMailboxConfView *view_info;
-    GtkWidget *box;
-    GtkWidget *button;
+    GtkWidget *widget;
     GList *list;
     const gchar *identity_name;
     gint active;
@@ -1339,8 +1364,12 @@ mailbox_conf_view_new(LibBalsaMailbox * mailbox,
                       (GWeakNotify) mailbox_conf_view_free, view_info);
     view_info->window = window;
 
-    create_label(_("Identity:"), table, row);
-    view_info->identity_combo_box = gtk_combo_box_new_text();
+    label = libbalsa_create_label(_("_Identity:"), table, row);
+    if (size_group)
+        gtk_size_group_add_widget(size_group, label);
+
+    view_info->identity_combo_box = widget = gtk_combo_box_new_text();
+    gtk_label_set_mnemonic_widget(GTK_LABEL(label), widget);
     identity_name = libbalsa_mailbox_get_identity_name(mailbox);
     for (list = balsa_app.identities, active = 0; list;
          list = list->next, ++active) {
@@ -1348,36 +1377,15 @@ mailbox_conf_view_new(LibBalsaMailbox * mailbox,
         gchar *name;
 
         name = internet_address_to_string(ident->ia, FALSE);
-        gtk_combo_box_append_text(GTK_COMBO_BOX
-                                  (view_info->identity_combo_box), name);
+        gtk_combo_box_append_text(GTK_COMBO_BOX(widget), name);
         g_free(name);
         if (identity_name
             && strcmp(identity_name, ident->identity_name) == 0)
-            gtk_combo_box_set_active(GTK_COMBO_BOX
-                                     (view_info->identity_combo_box),
-                                     active);
+            gtk_combo_box_set_active(GTK_COMBO_BOX(widget), active);
     }
-    gtk_table_attach(GTK_TABLE(table), view_info->identity_combo_box, 1, 2,
-                     row, row + 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 5);
-
-    create_label(_("Show address:"), table, ++row);
-    box = gtk_hbox_new(FALSE, 12);
-    view_info->show_from = button =
-        gtk_radio_button_new_with_mnemonic(NULL, _("_From"));
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
-    gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
-    view_info->show_to = button =
-        gtk_radio_button_new_with_mnemonic(gtk_radio_button_get_group
-                                           (GTK_RADIO_BUTTON(button)),
-                                           _("_To"));
-    gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
-    gtk_table_attach(GTK_TABLE(table), box, 1, 2, row, row + 1,
+    gtk_table_attach(GTK_TABLE(table), widget, 1, 2, row, row + 1,
                      GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
 
-    button = (libbalsa_mailbox_get_show(mailbox) == LB_MAILBOX_SHOW_TO ?
-	      view_info->show_to : view_info->show_from);
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
-
 #ifdef HAVE_GPGME
     {
 	/* scope */
@@ -1387,9 +1395,14 @@ mailbox_conf_view_new(LibBalsaMailbox * mailbox,
 	    { N_("Always"),      LB_MAILBOX_CHK_CRYPT_ALWAYS }
 	};
 	
-	create_label(_("Decrypt and check\nsignatures automatically:"), table, ++row);
-	
+        label =
+            libbalsa_create_label(_("_Decrypt and check\nsignatures automatically:"),
+                         table, ++row);
+        if (size_group)
+            gtk_size_group_add_widget(size_group, label);
+
         view_info->chk_crypt = gtk_combo_box_new_text();
+        gtk_label_set_mnemonic_widget(GTK_LABEL(label), view_info->chk_crypt);
         mailbox_conf_combo_box_make(GTK_COMBO_BOX(view_info->chk_crypt),
                                     ELEMENTS(chk_crypt_menu),
                                     chk_crypt_menu);
@@ -1402,9 +1415,31 @@ mailbox_conf_view_new(LibBalsaMailbox * mailbox,
     }
 #endif
 
+    /* Show address check button */
+    view_info->show_to =
+        libbalsa_create_check(_("Show _Recipient column instead of Sender"),
+                              table, ++row,
+                              libbalsa_mailbox_get_show(mailbox) ==
+                              LB_MAILBOX_SHOW_TO);
+    
+    /* Subscribe check button */
+    view_info->subscribe =
+        libbalsa_create_check(_("_Subscribe for new mail check"), table,
+                              ++row,
+                              libbalsa_mailbox_get_subscribe(mailbox) !=
+                              LB_MAILBOX_SUBSCRIBE_NO);
+
     return view_info;
 }
 
+BalsaMailboxConfView *
+mailbox_conf_view_new(LibBalsaMailbox * mailbox,
+                      GtkWindow * window, GtkWidget * table, gint row)
+{
+    return mailbox_conf_view_new_with_size_group(mailbox, window, table,
+                                                 row, NULL);
+}
+
 #ifdef HAVE_GPGME
 static LibBalsaChkCryptoMode
 balsa_mailbox_conf_get_crypto_mode(BalsaMailboxConfView *view_info)
@@ -1454,11 +1489,18 @@ mailbox_conf_view_check(BalsaMailboxConfView * view_info,
         changed = TRUE;
     }
 
-    if (libbalsa_mailbox_set_show(mailbox,
-				  gtk_toggle_button_get_active
-				  (GTK_TOGGLE_BUTTON(view_info->show_from))
-				  ? LB_MAILBOX_SHOW_FROM
-				  : LB_MAILBOX_SHOW_TO))
+    active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
+                                          (view_info->show_to));
+    if (libbalsa_mailbox_set_show(mailbox, active ?
+                                  LB_MAILBOX_SHOW_TO :
+                                  LB_MAILBOX_SHOW_FROM))
+        changed = TRUE;
+
+    active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
+                                          (view_info->subscribe));
+    if (libbalsa_mailbox_set_subscribe(mailbox, active ?
+				       LB_MAILBOX_SUBSCRIBE_YES :
+				       LB_MAILBOX_SUBSCRIBE_NO))
 	changed = TRUE;
 
 #ifdef HAVE_GPGME
