@@ -39,7 +39,11 @@
 #include <locale.h>
 #endif
 
-#include "i18n.h"
+#ifdef HAVE_RUBRICA
+#include <libxml/xmlversion.h>
+#endif
+
+#include <glib/gi18n.h>
 
 #include "address-entry.h"
 
@@ -60,8 +64,10 @@
 #include "Balsa.h"
 #include "balsa-bonobo.h"
 #include <bonobo-activation/bonobo-activation.h>
+#include <bonobo/bonobo-exception.h>
 
 #ifdef HAVE_GPGME
+#include <string.h>
 #include <gpgme.h>
 #endif
 
@@ -110,6 +116,23 @@ static GSList* opt_attach_list = NULL;
 /* opt_compose_email: To: field for the compose window */
 static gchar *opt_compose_email = NULL;
 
+static void
+accel_map_load(void)
+{
+    gchar *accel_map_filename =
+        g_build_filename(g_get_home_dir(), ".balsa", "accelmap", NULL);
+    gtk_accel_map_load(accel_map_filename);
+    g_free(accel_map_filename);
+}
+
+static void
+accel_map_save(void)
+{
+    gchar *accel_map_filename =
+        g_build_filename(g_get_home_dir(), ".balsa", "accelmap", NULL);
+    gtk_accel_map_save(accel_map_filename);
+    g_free(accel_map_filename);
+}
 
 static void
 balsa_handle_automation_options() {
@@ -124,10 +147,11 @@ balsa_handle_automation_options() {
        ("OAFIID:GNOME_Balsa_Application_Factory",
 	Bonobo_ACTIVATION_FLAG_EXISTING_ONLY,
 	NULL, &ev);
-
-   if (factory) {
+   
+   if ( !(BONOBO_EX (&ev) || factory == CORBA_OBJECT_NIL) ) {
        /* there already is a server. good */
        CORBA_Object app;
+       printf("Another Balsa found. Talking to it...\n");
        app =  
 	   bonobo_activation_activate_from_id ("OAFIID:GNOME_Balsa_Application",
 						   0, NULL, &ev);
@@ -411,7 +435,8 @@ threads_init(void)
     mail_thread_msg_receive =
 	g_io_channel_unix_new(mail_thread_pipes[0]);
     g_io_add_watch(mail_thread_msg_receive, G_IO_IN,
-		   (GIOFunc) mail_progress_notify_cb, NULL);
+		   (GIOFunc) mail_progress_notify_cb, 
+                   &balsa_app.main_window);
     
     if (pipe(send_thread_pipes) < 0) {
 	g_log("BALSA Init", G_LOG_LEVEL_DEBUG,
@@ -421,7 +446,8 @@ threads_init(void)
     send_thread_msg_receive =
 	g_io_channel_unix_new(send_thread_pipes[0]);
     g_io_add_watch(send_thread_msg_receive, G_IO_IN,
-		   (GIOFunc) send_progress_notify_cb, NULL);
+		   (GIOFunc) send_progress_notify_cb,
+                   &balsa_app.main_window);
 }
 
 static void
@@ -601,7 +627,7 @@ periodic_expunge_cb(void)
 /*
  * Initialize the progress bar and set text.
  */
-static gdouble prev_fraction;
+static GTimeVal prev_time_val;
 static void
 balsa_progress_set_text(LibBalsaProgress * progress, const gchar * text,
                         guint total)
@@ -617,7 +643,7 @@ balsa_progress_set_text(LibBalsaProgress * progress, const gchar * text,
 
     if (!text || total >= LIBBALSA_PROGRESS_MIN_COUNT)
         rc = balsa_window_setup_progress(balsa_app.main_window, text);
-    prev_fraction = 0;
+    g_get_current_time(&prev_time_val);
     gdk_threads_leave();
 
     *progress = (text && rc) ?
@@ -631,11 +657,18 @@ balsa_progress_set_text(LibBalsaProgress * progress, const gchar * text,
 static void
 balsa_progress_set_fraction(LibBalsaProgress * progress, gdouble fraction)
 {
+    GTimeVal time_val;
+    gdouble elapsed;
+
+    g_get_current_time(&time_val);
+    elapsed = time_val.tv_sec - prev_time_val.tv_sec;
+    elapsed += 1e-6 * (time_val.tv_usec - prev_time_val.tv_usec);
+
     if (*progress == LIBBALSA_PROGRESS_NO
         || (fraction != 0 && fraction != 1
-            && fraction < prev_fraction + LIBBALSA_PROGRESS_MIN_UPDATE))
+            && elapsed < LIBBALSA_PROGRESS_MIN_UPDATE))
         return;
-    prev_fraction = fraction;
+    prev_time_val = time_val;
 
     gdk_threads_enter();
     if (balsa_app.main_window)
@@ -644,6 +677,26 @@ balsa_progress_set_fraction(LibBalsaProgress * progress, gdouble fraction)
     gdk_threads_leave();
 }
 
+#if defined(ENABLE_NLS) && defined(HAVE_GPGME)
+static const gchar *
+get_utf8_locale(int category)
+{
+    gchar * locale;
+    static gchar localebuf[64];  /* should be large enough */
+    gchar * dot;
+
+    if (!(locale = setlocale(category, NULL)))
+	return NULL;
+    strncpy(localebuf, locale, 57);
+    localebuf[57] = '\0';
+    dot = strchr(localebuf, '.');
+    if (!dot)
+	dot = localebuf + strlen(localebuf);
+    strcpy(dot, ".UTF-8");
+    return localebuf;
+}
+#endif
+
 /* -------------------------- main --------------------------------- */
 int
 main(int argc, char *argv[])
@@ -663,19 +716,38 @@ main(int argc, char *argv[])
     setlocale(LC_ALL, "");
 #endif
 
+#ifdef BALSA_USE_THREADS
+    /* initiate thread mutexs, variables */
+    threads_init();
+#endif
+
+#ifdef HAVE_RUBRICA
+    /* initialise libxml */
+    LIBXML_TEST_VERSION
+#endif
 
 #ifdef HAVE_GPGME
     /* initialise the gpgme library */
     g_message("init gpgme version %s", gpgme_check_version(NULL));
-#ifdef ENABLE_NLS
-    gpgme_set_locale(NULL, LC_CTYPE, setlocale(LC_CTYPE, NULL));
-    gpgme_set_locale(NULL, LC_MESSAGES, setlocale(LC_MESSAGES, NULL));
-#endif /* ENABLE_NLS */
+#ifdef HAVE_GPG
+    /* configure the GnuPG engine if a specific gpg path has been detected*/
+    gpgme_set_engine_info(GPGME_PROTOCOL_OpenPGP, GPG_PATH, NULL);
 #endif
+#ifdef ENABLE_NLS
+    gpgme_set_locale(NULL, LC_CTYPE, get_utf8_locale(LC_CTYPE));
+    gpgme_set_locale(NULL, LC_MESSAGES, get_utf8_locale(LC_MESSAGES));
+#endif /* ENABLE_NLS */
+    { /* scope */
+	gpgme_engine_info_t e;
 
-#ifdef BALSA_USE_THREADS
-    /* initiate thread mutexs, variables */
-    threads_init();
+	if (gpgme_get_engine_info(&e) == GPG_ERR_NO_ERROR)
+	    while (e) {
+		g_message("protocol %s: engine %s (home %s, version %s)",
+			  gpgme_get_protocol_name(e->protocol),
+			  e->file_name, e->home_dir, e->version);
+		e = e->next;
+	    }
+    }
 #endif
 
     /* FIXME: do we need to allow a non-GUI mode? */
@@ -758,7 +830,7 @@ main(int argc, char *argv[])
         BalsaSendmsg *snd;
         GSList *lst;
         gdk_threads_enter();
-        snd = sendmsg_window_compose(window);
+        snd = sendmsg_window_compose();
         gdk_threads_leave();
         if(opt_compose_email) {
             if(g_ascii_strncasecmp(opt_compose_email, "mailto:", 7) == 0)
@@ -779,10 +851,12 @@ main(int argc, char *argv[])
     g_idle_add((GSourceFunc) scan_mailboxes_idle_cb, NULL);
     g_timeout_add(1801*1000, (GSourceFunc) periodic_expunge_cb, NULL);
 
+    accel_map_load();
     gdk_threads_enter();
     gtk_main();
     gdk_threads_leave();
-    
+    accel_map_save();
+
 #ifdef BALSA_USE_THREADS
     threads_destroy();
 #endif
