@@ -23,7 +23,7 @@
 
 #include <gnome.h>
 #include <string.h>
-#include "i18n.h"
+#include <glib/gi18n.h>
 
 #include "balsa-app.h"
 #include "store-address.h"
