@@ -23,28 +23,16 @@
 
 #include <string.h>
 #include <gnome.h>
-
-#include "libbalsa.h"
+#include <glib/gi18n.h>
 
 #include "balsa-app.h"
 #include "balsa-icons.h"
-#include "balsa-index.h"
-#include "balsa-mblist.h"
-#include "balsa-message.h"
-#include "folder-conf.h"
-#include "mailbox-conf.h"
 #include "main-window.h"
 #include "message-window.h"
-#include "pref-manager.h"
-#include "print.h"
 #include "sendmsg-window.h"
-#include "store-address.h"
-#include "save-restore.h"
+#include "toolbar-factory.h"
 #include "toolbar-prefs.h"
 
-
-#include "libinit_balsa/init_balsa.h"
-
 #ifndef MAX
 #define MAX(a, b) ((a) > (b) ? (a) : (b))
 #endif
@@ -73,7 +61,6 @@ struct ToolbarPage_ {
     GtkWidget *back_button;
     GtkWidget *forward_button;
     GtkWidget *standard_button;
-    gboolean needs_refresh;
 };
 
 /* Callbacks. */
@@ -98,7 +85,8 @@ static void current_row_activated_cb(GtkTreeView * treeview,
                                      ToolbarPage * page);
 
 /* Helpers. */
-static GtkWidget *create_toolbar_page(BalsaToolbarModel * model);
+static GtkWidget *create_toolbar_page(BalsaToolbarModel * model,
+                                      GtkUIManager * ui_manager);
 static GtkWidget *tp_list_new(void);
 static gboolean tp_list_iter_is_first(GtkWidget * list, GtkTreeIter * iter);
 static gboolean tp_list_iter_is_last(GtkWidget * list, GtkTreeIter * iter);
@@ -124,7 +112,10 @@ customize_dialog_cb(GtkWidget * widget, gpointer data)
     GtkWidget *option_frame;
     GtkWidget *option_box;
     GtkWidget *wrap_button;
-    GnomeApp *active_window = GNOME_APP(data);
+    GtkWidget *active_window = data;
+    BalsaToolbarModel *model;
+    BalsaToolbarType   type;
+    GtkUIManager * ui_manager;
 
     /* There can only be one */
     if (customize_widget) {
@@ -152,15 +143,23 @@ customize_dialog_cb(GtkWidget * widget, gpointer data)
                            "Balsa");
     gtk_window_set_default_size(GTK_WINDOW(customize_widget), 600, 440);
 
-    child = create_toolbar_page(balsa_window_get_toolbar_model());
+    /* The order of pages must be consistent with the BalsaToolbarType
+     * enum. */
+    model = balsa_window_get_toolbar_model(&ui_manager);
+    child = create_toolbar_page(model, ui_manager);
+    g_object_unref(ui_manager);
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), child,
                              gtk_label_new(_("Main window")));
 
-    child = create_toolbar_page(sendmsg_window_get_toolbar_model());
+    model = sendmsg_window_get_toolbar_model(&ui_manager);
+    child = create_toolbar_page(model, ui_manager);
+    g_object_unref(ui_manager);
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), child,
                              gtk_label_new(_("Compose window")));
 
-    child = create_toolbar_page(message_window_get_toolbar_model());
+    model = message_window_get_toolbar_model(&ui_manager);
+    child = create_toolbar_page(model, ui_manager);
+    g_object_unref(ui_manager);
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), child,
                              gtk_label_new(_("Message window")));
 
@@ -182,6 +181,13 @@ customize_dialog_cb(GtkWidget * widget, gpointer data)
     gtk_box_pack_start(GTK_BOX(option_box), wrap_button, FALSE, FALSE, 0);
 
     gtk_widget_show_all(customize_widget);
+
+    /* Now that the pages are shown, we can switch to the page
+     * corresponding to the toolbar that the user clicked on. */
+    type =
+        GPOINTER_TO_INT(g_object_get_data
+                        (G_OBJECT(widget), BALSA_TOOLBAR_MODEL_TYPE));
+    gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), type);
 }
 
 /* get_toolbar_button_index:
@@ -197,7 +203,7 @@ get_toolbar_button_index(const char *id)
         gchar *old;
     } button_converter[] = {
         { BALSA_PIXMAP_ATTACHMENT,   GNOME_STOCK_ATTACH },
-        { BALSA_PIXMAP_NEW,          GNOME_STOCK_MAIL_NEW },
+        { BALSA_PIXMAP_COMPOSE,      GNOME_STOCK_MAIL_NEW },
         { BALSA_PIXMAP_CONTINUE,     GNOME_STOCK_MAIL },
         { BALSA_PIXMAP_RECEIVE,      GNOME_STOCK_MAIL_RCV },
         { BALSA_PIXMAP_REPLY,        GNOME_STOCK_MAIL_RPL },
@@ -206,20 +212,19 @@ get_toolbar_button_index(const char *id)
         { BALSA_PIXMAP_FORWARD,      GNOME_STOCK_MAIL_FWD },
         { BALSA_PIXMAP_NEXT,         GTK_STOCK_GO_FORWARD },
         { BALSA_PIXMAP_PREVIOUS,     GTK_STOCK_GO_BACK },
-        { BALSA_PIXMAP_PRINT,        GTK_STOCK_PRINT },
-        { BALSA_PIXMAP_SAVE,         GTK_STOCK_SAVE },
+	{ GTK_STOCK_PRINT,           BALSA_OLD_PIXMAP_PRINT },
+	{ GTK_STOCK_SAVE,            BALSA_OLD_PIXMAP_SAVE },
         { BALSA_PIXMAP_SEND,         GNOME_STOCK_MAIL_SND },
-        { BALSA_PIXMAP_TRASH,        GNOME_STOCK_TRASH },
+	{ GTK_STOCK_DELETE,          BALSA_OLD_PIXMAP_TRASH },
         { BALSA_PIXMAP_TRASH_EMPTY,  "empty_trash" },
         { BALSA_PIXMAP_NEXT_UNREAD,  "next_unread" },
         { BALSA_PIXMAP_NEXT_FLAGGED, "next_flagged" },
         { BALSA_PIXMAP_SHOW_HEADERS, "show_all_headers" },
         { BALSA_PIXMAP_SHOW_PREVIEW, "show_preview" },
         { BALSA_PIXMAP_MARKED_NEW,   "flag_unread" },
-        { BALSA_PIXMAP_MARKED_ALL,   "mark_all" },
+        { BALSA_PIXMAP_MARK_ALL,     "mark_all" },
         { BALSA_PIXMAP_IDENTITY,     "identity" },
-        { BALSA_PIXMAP_CLOSE_MBOX,   GTK_STOCK_CLOSE },
-        { BALSA_PIXMAP_OTHER_CLOSE,  "close_mbox" },
+	{ GTK_STOCK_CANCEL,          BALSA_OLD_PIXMAP_CLOSE_MBOX },
         { NULL, NULL }
     };
 #endif
@@ -257,16 +262,15 @@ get_toolbar_button_index(const char *id)
 static void
 wrap_toggled_cb(GtkWidget * widget, GtkNotebook * notebook)
 {
-    gint i, j;
+    gint i;
     GtkWidget *child;
     ToolbarPage *page;
     
     balsa_app.toolbar_wrap_button_text = GTK_TOGGLE_BUTTON(widget)->active;
 
-    j = gtk_notebook_get_current_page(notebook);
     for (i = 0; (child = gtk_notebook_get_nth_page(notebook, i)); i++) {
         page = g_object_get_data(G_OBJECT(child), BALSA_KEY_TOOLBAR_PAGE);
-        balsa_toolbar_refresh(page->toolbar);
+        balsa_toolbar_model_changed(page->model);
     }
 }
 
@@ -303,9 +307,18 @@ static void
 standard_button_cb(GtkWidget *widget, ToolbarPage * page)
 {
     balsa_toolbar_model_clear(page->model);
-    balsa_toolbar_refresh(page->toolbar);
+    gtk_widget_set_sensitive(page->standard_button, FALSE);
     tp_page_refresh_available(page);
     tp_page_refresh_current(page);
+    balsa_toolbar_model_changed(page->model);
+}
+
+static void
+style_button_cb(GtkWidget *widget, ToolbarPage * page)
+{
+    gboolean handled;
+
+    g_signal_emit_by_name(page->toolbar, "popup-menu", &handled);
 }
 
 /* Callback for the "row-activated" signal for the available list. */
@@ -367,7 +380,6 @@ tp_dialog_response_cb(GtkDialog * dialog, gint response, gpointer data)
     switch (response) {
     case GTK_RESPONSE_DELETE_EVENT:
     case GTK_RESPONSE_CLOSE:
-        update_all_toolbars();
         gtk_widget_destroy(GTK_WIDGET(dialog));
         break;
     case GTK_RESPONSE_HELP:
@@ -389,7 +401,7 @@ tp_dialog_response_cb(GtkDialog * dialog, gint response, gpointer data)
 /* Create a page for the main notebook.
  */
 static GtkWidget*
-create_toolbar_page(BalsaToolbarModel * model)
+create_toolbar_page(BalsaToolbarModel * model, GtkUIManager * ui_manager)
 {
     GtkWidget *outer_box;
     GtkWidget *toolbar_frame, *toolbar_scroll;
@@ -397,16 +409,17 @@ create_toolbar_page(BalsaToolbarModel * model)
     GtkWidget *lower_ctlbox, *button_box, *move_button_box, *center_button_box;
     GtkWidget *list_frame, *list_scroll;
     GtkWidget *destination_frame, *destination_scroll;
+    GtkWidget *style_button;
     ToolbarPage *page;
     GtkTreeSelection *selection;
 
     page = g_new(ToolbarPage, 1);
     page->model = model;
-    page->needs_refresh = FALSE;
 
     /* The "window itself" */
     outer_box=gtk_vbox_new(FALSE, 0);
-    g_object_set_data(G_OBJECT(outer_box), BALSA_KEY_TOOLBAR_PAGE, page);
+    g_object_set_data_full(G_OBJECT(outer_box), BALSA_KEY_TOOLBAR_PAGE,
+                           page, g_free);
 
     /* Preview display */
     toolbar_frame=gtk_frame_new(_("Preview"));
@@ -417,11 +430,16 @@ create_toolbar_page(BalsaToolbarModel * model)
     gtk_container_add(GTK_CONTAINER(toolbar_frame), toolbar_ctlbox);
     gtk_container_set_border_width(GTK_CONTAINER(toolbar_ctlbox), 5);
 
+    /* The ui-manager has actions but no ui, so we add an empty toolbar. */
+    gtk_ui_manager_add_ui_from_string(ui_manager,
+                                      "<ui>"
+                                      "  <toolbar name='Toolbar'/>"
+                                      "</ui>",
+                                      -1, NULL);
+
     /* The preview is an actual, fully functional toolbar */
-    page->toolbar = balsa_toolbar_new(model);
-    g_object_add_weak_pointer(G_OBJECT(page->toolbar),
-                              (gpointer) & page->toolbar);
-    gtk_toolbar_set_style(GTK_TOOLBAR(page->toolbar), GTK_TOOLBAR_BOTH);
+    page->toolbar = balsa_toolbar_new(model, ui_manager);
+    gtk_widget_set_sensitive(page->toolbar, FALSE);
 
     /* embedded in a scrolled_window */
     toolbar_scroll=gtk_scrolled_window_new(NULL, NULL);
@@ -435,12 +453,18 @@ create_toolbar_page(BalsaToolbarModel * model)
     gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(toolbar_scroll),
                                           page->toolbar);
 
+    /* Button box */
+    button_box = gtk_hbutton_box_new();
+    gtk_box_pack_start(GTK_BOX(toolbar_ctlbox), button_box, FALSE, FALSE, 0);
+
     /* Standard button */
     page->standard_button =
         gtk_button_new_with_mnemonic(_("_Restore toolbar to standard buttons"));
+    gtk_container_add(GTK_CONTAINER(button_box), page->standard_button);
 
-    gtk_box_pack_start(GTK_BOX(toolbar_ctlbox), page->standard_button,
-		       FALSE, FALSE, 0);
+    /* Style button */
+    style_button = gtk_button_new_with_mnemonic(_("Toolbar _style..."));
+    gtk_container_add(GTK_CONTAINER(button_box), style_button);
 
     /* Done with preview */
 	
@@ -537,11 +561,15 @@ create_toolbar_page(BalsaToolbarModel * model)
 
     g_signal_connect(G_OBJECT(page->standard_button), "clicked",
 		       G_CALLBACK(standard_button_cb), page);
+    g_signal_connect(G_OBJECT(style_button), "clicked",
+		       G_CALLBACK(style_button_cb), page);
 
     gtk_widget_set_sensitive(page->add_button, FALSE);
     gtk_widget_set_sensitive(page->remove_button, FALSE);
     gtk_widget_set_sensitive(page->back_button, FALSE);
     gtk_widget_set_sensitive(page->forward_button, FALSE);
+    gtk_widget_set_sensitive(page->standard_button,
+                             !balsa_toolbar_model_is_standard(model));
 
     tp_page_refresh_available(page);
     tp_page_refresh_current(page);
@@ -559,7 +587,7 @@ tp_page_refresh_available(ToolbarPage * page)
     GtkTreeModel *model;
     GtkTreeIter iter;
     GtkTreePath *path;
-    GSList *legal = balsa_toolbar_model_get_legal(page->model);
+    GHashTable *legal = balsa_toolbar_model_get_legal(page->model);
     GSList *current = balsa_toolbar_model_get_current(page->model);
     int item;
 
@@ -574,9 +602,8 @@ tp_page_refresh_available(ToolbarPage * page)
 
     for (item = 0; item < toolbar_button_count; item++) {
         if (item > 0
-            && (!g_slist_find_custom(legal,
-                                     toolbar_buttons[item].pixmap_id,
-                                     (GCompareFunc) strcmp)
+            && (!g_hash_table_lookup(legal,
+                                     toolbar_buttons[item].pixmap_id)
                 || g_slist_find_custom(current,
                                        toolbar_buttons[item].pixmap_id,
                                        (GCompareFunc) strcmp)))
@@ -658,7 +685,6 @@ tp_page_refresh_preview(ToolbarPage * page)
                                             toolbar_buttons[item].pixmap_id,
                                             -1);
     }
-    balsa_toolbar_refresh(page->toolbar);
 }
 
 /* Create a GtkTreeView for an icon list.
@@ -762,7 +788,9 @@ tp_page_swap_rows(ToolbarPage * page, gboolean forward)
                                  tmp_path, NULL, FALSE, 0, 0);
     gtk_tree_path_free(tmp_path);
 
+    gtk_widget_set_sensitive(page->standard_button, TRUE);
     tp_page_refresh_preview(page);
+    balsa_toolbar_model_changed(page->model);
 }
 
 /* Add an item to a GtkTreeView's GtkListStore.
@@ -773,7 +801,7 @@ tp_store_set(GtkListStore * store, GtkTreeIter * iter, gint item)
     GdkPixbuf *pixbuf;
     gchar *text;
 
-    text = g_strdup(_(toolbar_buttons[item].button_text));
+    text = g_strdup(balsa_toolbar_button_text(item));
     replace_nl_with_space(text);
     pixbuf = (item > 0
               ? gtk_widget_render_icon(customize_widget,
@@ -824,8 +852,10 @@ tp_page_add_selected(ToolbarPage * page)
                                  FALSE, 0, 0);
     gtk_tree_path_free(path);
 
+    gtk_widget_set_sensitive(page->standard_button, TRUE);
     tp_page_refresh_preview(page);
     tp_page_refresh_available(page);
+    balsa_toolbar_model_changed(page->model);
 }
 
 /* Remove an item from the page's current list.
@@ -853,8 +883,10 @@ tp_page_remove_selected(ToolbarPage * page)
     }
     gtk_tree_path_free(path);
 
+    gtk_widget_set_sensitive(page->standard_button, TRUE);
     tp_page_refresh_preview(page);
     tp_page_refresh_available(page);
+    balsa_toolbar_model_changed(page->model);
 }
 
 /* Unwrap text.
